#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     November 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a GUI for canvas

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
import numpy as np
import vispy.io as vispy_io
from vispy import scene
from vispy.color import Colormap
from functools import partial
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.vis.colormap import colormap as vis_cmap
from cognitivegeo.src.gui.configseisvis import configseisvis as gui_configseisvis
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

class plotviscanvas(object):

    survinfo = {}
    seisdata = {}
    pointsetdata = {}
    settings = core_set.Settings
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    canvas = None
    view = None
    canvasproperties = {}
    canvascomponents = {}
    seisvisconfig = {}


    def setupGUI(self, PlotVisCanvas):
        PlotVisCanvas.setObjectName("PlotVisCanvas")
        PlotVisCanvas.setFixedSize(1400, 960)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/canvas.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVisCanvas.setWindowIcon(icon)
        #
        # seismic
        self.lblseis = QtWidgets.QLabel(PlotVisCanvas)
        self.lblseis.setObjectName("lblseis")
        self.lblseis.setGeometry(QtCore.QRect(10, 10, 50, 30))
        # config seismic
        self.btnconfigseisvis = QtWidgets.QPushButton(PlotVisCanvas)
        self.btnconfigseisvis.setObjectName("btnconfigseisvis")
        self.btnconfigseisvis.setGeometry(QtCore.QRect(60, 10, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/settings.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigseisvis.setIcon(icon)
        # seismic list
        self.cbblistseis = QtWidgets.QComboBox(PlotVisCanvas)
        self.cbblistseis.setObjectName("cbblistseis")
        self.cbblistseis.setGeometry(QtCore.QRect(10, 50, 240, 30))
        # add to canvas
        self.btnaddseis2canvas = QtWidgets.QPushButton(PlotVisCanvas)
        self.btnaddseis2canvas.setObjectName("btnaddseis2canvas")
        self.btnaddseis2canvas.setGeometry(QtCore.QRect(260, 50, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/add.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnaddseis2canvas.setIcon(icon)
        # seismic on canvas
        self.twgseisoncanvas = QtWidgets.QTableWidget(PlotVisCanvas)
        self.twgseisoncanvas.setObjectName("twgseisoncanvas")
        self.twgseisoncanvas.setGeometry(QtCore.QRect(10, 90, 280, 300))
        self.twgseisoncanvas.setColumnCount(5)
        self.twgseisoncanvas.setRowCount(0)
        self.twgseisoncanvas.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgseisoncanvas.horizontalHeader().hide()
        self.twgseisoncanvas.verticalHeader().hide()
        self.twgseisoncanvas.setColumnWidth(0, 18)
        self.twgseisoncanvas.setColumnWidth(1, 108)
        self.twgseisoncanvas.setColumnWidth(2, 60)
        self.twgseisoncanvas.setColumnWidth(3, 60)
        self.twgseisoncanvas.setColumnWidth(4, 30)
        #
        # main canvas
        self.wgtcanvas = QtWidgets.QWidget(PlotVisCanvas)
        self.wgtcanvas.setObjectName("wdtcanvas")
        self.wgtcanvas.setGeometry(QtCore.QRect(300, 50, 1050, 900))
        #
        # survey box
        self.btnsrvbox = QtWidgets.QPushButton(PlotVisCanvas)
        self.btnsrvbox.setObjectName("btnsrvbox")
        self.btnsrvbox.setGeometry(QtCore.QRect(1360, 50, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/box.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnsrvbox.setIcon(icon)
        #
        # xyz axis
        self.btnxyzaxis = QtWidgets.QPushButton(PlotVisCanvas)
        self.btnxyzaxis.setObjectName("btnxyzaxis")
        self.btnxyzaxis.setGeometry(QtCore.QRect(1360, 90, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/xyz.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnxyzaxis.setIcon(icon)
        #
        # snapshot
        self.btnsnapshot = QtWidgets.QPushButton(PlotVisCanvas)
        self.btnsnapshot.setObjectName("btnsnapshot")
        self.btnsnapshot.setGeometry(QtCore.QRect(1360, 130, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/camera.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnsnapshot.setIcon(icon)
        #
        # home
        self.btngohome = QtWidgets.QPushButton(PlotVisCanvas)
        self.btngohome.setObjectName("btngohome")
        self.btngohome.setGeometry(QtCore.QRect(300, 10, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/home.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btngohome.setIcon(icon)
        #
        # view angle
        self.cbbviewfrom = QtWidgets.QComboBox(PlotVisCanvas)
        self.cbbviewfrom.setObjectName("cbbviewfrom")
        self.cbbviewfrom.setGeometry(QtCore.QRect(340, 10, 90, 30))
        #
        # z scale
        self.ldtzscale = QtWidgets.QLineEdit(PlotVisCanvas)
        self.ldtzscale.setObjectName("ldtzscale")
        self.ldtzscale.setGeometry(QtCore.QRect(440, 10, 60, 30))
        self.ldtzscale.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVisCanvas)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVisCanvas.geometry().center().x()
        _center_y = PlotVisCanvas.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVisCanvas)
        QtCore.QMetaObject.connectSlotsByName(PlotVisCanvas)


    def retranslateGUI(self, PlotVisCanvas):
        self.dialog = PlotVisCanvas
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVisCanvas.setWindowTitle(_translate("PlotVisCanvas", "Canvas"))
        #
        # seismic
        self.lblseis.setText(_translate("PlotVisCanvas", "Seismic:"))
        # seismic configuration
        self.btnconfigseisvis.setText(_translate("PlotVisCanvas", ""))
        self.btnconfigseisvis.clicked.connect(self.clickBtnConfigSeisVis)
        # seismic list
        self.cbblistseis.addItems(list(self.seisdata.keys()))
        # add seismic to canvas
        self.btnaddseis2canvas.setText(_translate("PlotVisCanvas", ""))
        self.btnaddseis2canvas.setToolTip("Add to canvas")
        self.btnaddseis2canvas.clicked.connect(self.clickBtnAddSeis2Canvas)
        # seismic on canvas
        # self.twgseisoncanvas.currentCellChanged.connect(self.changeTwgSeisOnCanvas)
        #
        # main canvas
        self.canvas = scene.SceneCanvas(keys='interactive', title='Canvas', bgcolor=[0.5, 0.5, 0.5],
                                        size=(1080, 920), app='pyqt5', parent=self.wgtcanvas)
        self.view = self.canvas.central_widget.add_view()
        self.view.camera = scene.TurntableCamera(elevation=30, azimuth=135)
        #
        # survey box
        self.btnsrvbox.setText(_translate("PlotVisCanvas", ""))
        self.btnsrvbox.setToolTip("Survey box")
        self.btnsrvbox.setDefault(False)
        self.btnsrvbox.clicked.connect(self.clickBtnSrvBox)
        #
        # xyz axis
        self.btnxyzaxis.setText(_translate("PlotVisCanvas", ""))
        self.btnxyzaxis.setToolTip("XYZ axis")
        self.btnxyzaxis.setDefault(False)
        self.btnxyzaxis.clicked.connect(self.clickBtnXYZAxis)
        #
        # snapshot
        self.btnsnapshot.setText(_translate("PlotVisCanvas", ""))
        self.btnsnapshot.setToolTip("Snapshot")
        self.btnsnapshot.setDefault(False)
        self.btnsnapshot.clicked.connect(self.clickBtnSnapshot)
        #
        # home
        self.btngohome.setText(_translate("PlotVisCanvas", ""))
        self.btngohome.setDefault(True)
        self.btngohome.setToolTip("Home view")
        self.btngohome.clicked.connect(self.clickBtnGoHome)
        #
        # view angle
        self.cbbviewfrom.addItems(['Inline', 'Crossline', 'Top'])
        self.cbbviewfrom.setItemIcon(0, QtGui.QIcon(
                QtGui.QPixmap(os.path.join(self.iconpath, "icons/visinl.png")).scaled(30, 30)))
        self.cbbviewfrom.setItemIcon(1, QtGui.QIcon(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/visxl.png")).scaled(30, 30)))
        self.cbbviewfrom.setItemIcon(2, QtGui.QIcon(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/visz.png")).scaled(30, 30)))
        self.cbbviewfrom.setCurrentIndex(0)
        self.cbbviewfrom.currentIndexChanged.connect(self.changeCbbViewFrom)
        #
        # z scale
        self.ldtzscale.setText(_translate("PlotVisCanvas", "1.0"))
        self.ldtzscale.textChanged.connect(self.changeLdtZScale)
        #
        self.initSeisVis()
        self.initCanvas()


    def initSeisVis(self):
        self.seisvisconfig = {}
        if len(self.seisdata.keys()) > 0:
            for seis in self.seisdata.keys():
                if self.checkSeisData(seis):
                    self.seisvisconfig[seis] = {}
                    self.seisvisconfig[seis]['Colormap'] = self.settings['Visual']['Image']['Colormap']
                    self.seisvisconfig[seis]['Flip'] = False
                    self.seisvisconfig[seis]['Opacity'] = '100%'
                    self.seisvisconfig[seis]['Interpolation'] = self.settings['Visual']['Image']['Interpolation']
                    self.seisvisconfig[seis]['Maximum'] = np.max(self.seisdata[seis])
                    self.seisvisconfig[seis]['Minimum'] = np.min(self.seisdata[seis])


    def initCanvas(self):
        #
        # components and properties
        self.canvasproperties['Seismic'] = []
        self.canvascomponents['Seismic'] = []
        self.canvasproperties['Survey_Box'] = False
        self.canvascomponents['Survey_Box'] = []
        self.canvasproperties['XYZ_Axis'] = False
        self.canvascomponents['XYZ_Axis'] = None
        self.canvasproperties['Z_Scale'] = 1.0
        #
        # add survey box
        self.canvascomponents['Survey_Box'] = self.createSrvBox()
        if len(self.canvascomponents['Survey_Box']) == 12:
            for _i in self.canvascomponents['Survey_Box']:
                _i.parent = self.view.scene
            self.canvasproperties['Survey_Box'] = True
        #
        # add xyz axis
        self.canvascomponents['XYZ_Axis'] = self.createXYZAxis()
        if self.canvascomponents['XYZ_Axis'] is not None:
            self.canvascomponents['XYZ_Axis'].parent = self.view.scene
            self.canvasproperties['XYZ_Axis'] = True
        #
        # z scale
        _zscale = basic_data.str2float(self.ldtzscale.text())
        if _zscale is not False and _zscale > 0.0:
            self.canvasproperties['Z_Scale'] = _zscale
        #
        # set camara range
        self.setCameraRange()
        #
        self.canvas.show()


    def clickBtnConfigSeisVis(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configseisvis()
        _gui.seisvisconfig = self.seisvisconfig
        _gui.setupGUI(_config)
        _config.exec()
        self.seisvisconfig = _gui.seisvisconfig
        _config.show()
        #
        # update seismic
        for _i in range(len(self.canvascomponents['Seismic'])):
            self.canvascomponents['Seismic'][_i].parent = None
            _vis = self.createVisualSeis(self.canvasproperties['Seismic'][_i]['Name'],
                                         self.canvasproperties['Seismic'][_i]['Orientation'],
                                         self.canvasproperties['Seismic'][_i]['Number'])
            if self.canvasproperties['Seismic'][_i]['Visible']:
                _vis.parent = self.view.scene
            self.canvascomponents['Seismic'][_i] = _vis


    def clickBtnAddSeis2Canvas(self):
        if len(self.seisdata.keys()) > 0 and \
                self.checkSeisData(list(self.seisdata.keys())[self.cbblistseis.currentIndex()]):
            ##### property
            _seis = {}
            _seis['Visible'] = False
            _seis['Name'] = list(self.seisdata.keys())[self.cbblistseis.currentIndex()]
            _seis['Orientation'] = 'Inline'
            _seis['Number'] = self.survinfo['ILStart']
            _seis['Remove'] = False
            #
            self.canvasproperties['Seismic'].append(_seis)
            #
            _nseis = len(self.canvasproperties['Seismic'])
            # add one more row
            self.twgseisoncanvas.setRowCount(_nseis)
            # visible
            _item = QtWidgets.QCheckBox()
            _item.setChecked(_seis['Visible'])
            _item.stateChanged.connect(partial(self.changeCbxVisSeisOnCanvas, idx=_nseis - 1))
            self.twgseisoncanvas.setCellWidget(_nseis - 1, 0, _item)
            # name
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_seis['Name'])
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            self.twgseisoncanvas.setItem(_nseis - 1, 1, _item)
            # orientation
            _item = QtWidgets.QComboBox()
            _item.addItems(['Inline', 'Xline', 'Z'])
            _item.setCurrentIndex(list.index(['Inline', 'Crossline', 'Z'], _seis['Orientation']))
            _item.currentIndexChanged.connect(partial(self.changeCbbOrtSeisOnCanvas, idx=_nseis - 1))
            self.twgseisoncanvas.setCellWidget(_nseis - 1, 2, _item)
            # number
            _item = QtWidgets.QComboBox()
            _slices = []
            if _seis['Orientation'] == 'Inline':
                _slices = [str(_no) for _no in self.survinfo['ILRange']]
            if _seis['Orientation'] == 'Crossline':
                _slices = [str(_no) for _no in self.survinfo['XLRange']]
            if _seis['Orientation'] == 'Z':
                _slices = [str(_no) for _no in self.survinfo['ZRange']]
            _item.addItems(_slices)
            _item.setCurrentIndex(list.index(_slices, str(_seis['Number'])))
            _item.currentIndexChanged.connect(partial(self.changeCbbNoSeisOnCanvas, idx=_nseis - 1))
            self.twgseisoncanvas.setCellWidget(_nseis - 1, 3, _item)
            # remove
            _item = QtWidgets.QPushButton()
            _icon = QtGui.QIcon()
            _icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/no.png")),
                            QtGui.QIcon.Normal,
                            QtGui.QIcon.Off)
            _item.setIcon(_icon)
            _item.clicked.connect(partial(self.clickBtnRmSeisOnCanvas, idx=_nseis - 1))
            self.twgseisoncanvas.setCellWidget(_nseis - 1, 4, _item)
            #
            ### component
            _vis = self.createVisualSeis(_seis['Name'], _seis['Orientation'], _seis['Number'])
            if _seis['Visible']:
                _vis.parent = self.view.scene
            self.canvascomponents['Seismic'].append(_vis)


    def changeCbxVisSeisOnCanvas(self, idx):
        if idx < len(self.canvasproperties['Seismic']):
           self.canvasproperties['Seismic'][idx]['Visible'] = self.twgseisoncanvas.cellWidget(idx, 0).isChecked()
           #
           if self.twgseisoncanvas.cellWidget(idx, 0).isChecked():
               self.canvascomponents['Seismic'][idx].parent = self.view.scene
           else:
               self.canvascomponents['Seismic'][idx].parent = None
           #
           # print(idx)
           # print(self.canvasproperties['Seismic'][idx])


    def changeCbbOrtSeisOnCanvas(self, idx):
        if idx < len(self.canvasproperties['Seismic']):
            self.twgseisoncanvas.cellWidget(idx, 3).clear()
            _ort = ""
            _no = 0
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 0:
                self.twgseisoncanvas.cellWidget(idx, 3).addItems([str(_no) for _no in self.survinfo['ILRange']])
                self.twgseisoncanvas.cellWidget(idx, 3).setCurrentIndex(0)
                _ort = 'Inline'
                _no = self.survinfo['ILRange'][0]
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 1:
                self.twgseisoncanvas.cellWidget(idx, 3).addItems([str(_no) for _no in self.survinfo['XLRange']])
                self.twgseisoncanvas.cellWidget(idx, 3).setCurrentIndex(0)
                _ort = 'Crossline'
                _no = self.survinfo['XLRange'][0]
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 2:
                self.twgseisoncanvas.cellWidget(idx, 3).addItems([str(_no) for _no in self.survinfo['ZRange']])
                self.twgseisoncanvas.cellWidget(idx, 3).setCurrentIndex(0)
                _ort = 'Z'
                _no = self.survinfo['ZRange'][0]
            #
            self.canvasproperties['Seismic'][idx]['Orientation'] = _ort
            self.canvasproperties['Seismic'][idx]['Number'] = _no
            #
            self.canvascomponents['Seismic'][idx].parent = None
            _vis = self.createVisualSeis(self.canvasproperties['Seismic'][idx]['Name'], _ort, _no)
            if self.canvasproperties['Seismic'][idx]['Visible']:
                _vis.parent = self.view.scene
            self.canvascomponents['Seismic'][idx] = _vis


    def changeCbbNoSeisOnCanvas(self, idx):
        if idx < len(self.canvasproperties['Seismic']):
            _no = 0
            _ort = ''
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 0:
                _ort = 'Inline'
                _no = self.survinfo['ILRange'][self.twgseisoncanvas.cellWidget(idx, 3).currentIndex()]
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 1:
                _ort = 'Crossline'
                _no = self.survinfo['XLRange'][self.twgseisoncanvas.cellWidget(idx, 3).currentIndex()]
            if self.twgseisoncanvas.cellWidget(idx, 2).currentIndex() == 2:
                _ort = 'Z'
                _no = self.survinfo['ZRange'][self.twgseisoncanvas.cellWidget(idx, 3).currentIndex()]
            #
            self.canvasproperties['Seismic'][idx]['Number'] = _no
            #
            self.canvascomponents['Seismic'][idx].parent = None
            _vis = self.createVisualSeis(self.canvasproperties['Seismic'][idx]['Name'], _ort, _no)
            if self.canvasproperties['Seismic'][idx]['Visible']:
                _vis.parent = self.view.scene
            self.canvascomponents['Seismic'][idx] = _vis


    def clickBtnRmSeisOnCanvas(self, idx):
        if idx < len(self.canvasproperties['Seismic']):
            self.twgseisoncanvas.clear()
            self.canvasproperties['Seismic'].pop(idx)
            self.canvascomponents['Seismic'][idx].parent = None
            self.canvascomponents['Seismic'].pop(idx)
            self.updateTwgSeisOnCanvas()


    def updateTwgSeisOnCanvas(self):
        self.twgseisoncanvas.clear()
        #
        _seis = self.canvasproperties['Seismic']
        #
        self.twgseisoncanvas.setRowCount(len(_seis))
        for _i in range(len(_seis)):
            # visible
            _item = QtWidgets.QCheckBox()
            _item.setChecked(_seis[_i]['Visible'])
            _item.stateChanged.connect(partial(self.changeCbxVisSeisOnCanvas, idx=_i))
            self.twgseisoncanvas.setCellWidget(_i, 0, _item)
            # name
            _item = QtWidgets.QTableWidgetItem()
            _item.setText(_seis[_i]['Name'])
            _item.setTextAlignment(QtCore.Qt.AlignCenter)
            _item.setFlags(QtCore.Qt.ItemIsEditable)
            self.twgseisoncanvas.setItem(_i, 1, _item)
            # orientation
            _item = QtWidgets.QComboBox()
            _item.addItems(['Inline', 'Xline', 'Z'])
            _item.setCurrentIndex(list.index(['Inline', 'Crossline', 'Z'], _seis[_i]['Orientation']))
            _item.currentIndexChanged.connect(partial(self.changeCbbOrtSeisOnCanvas, idx=_i))
            self.twgseisoncanvas.setCellWidget(_i, 2, _item)
            # number
            _item = QtWidgets.QComboBox()
            _slices = []
            if _seis[_i]['Orientation'] == 'Inline':
                _slices = [str(_no) for _no in self.survinfo['ILRange']]
            if _seis[_i]['Orientation'] == 'Crossline':
                _slices = [str(_no) for _no in self.survinfo['XLRange']]
            if _seis[_i]['Orientation'] == 'Z':
                _slices = [str(_no) for _no in self.survinfo['ZRange']]
            _item.addItems(_slices)
            _item.setCurrentIndex(list.index(_slices, str(_seis[_i]['Number'])))
            _item.currentIndexChanged.connect(partial(self.changeCbbNoSeisOnCanvas, idx=_i))
            self.twgseisoncanvas.setCellWidget(_i, 3, _item)
            # remove
            _item = QtWidgets.QPushButton()
            _icon = QtGui.QIcon()
            _icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/no.png")),
                            QtGui.QIcon.Normal,
                            QtGui.QIcon.Off)
            _item.setIcon(_icon)
            _item.clicked.connect(partial(self.clickBtnRmSeisOnCanvas, idx=_i))
            self.twgseisoncanvas.setCellWidget(_i, 4, _item)


    def clickBtnSrvBox(self):
        if len(self.canvascomponents['Survey_Box']) == 12:
            if self.canvasproperties['Survey_Box'] is True:
                for _i in self.canvascomponents['Survey_Box']:
                    _i.parent = None
                self.canvasproperties['Survey_Box'] = False
            else:
                for _i in self.canvascomponents['Survey_Box']:
                    _i.parent = self.view.scene
                self.canvasproperties['Survey_Box'] = True


    def clickBtnXYZAxis(self):
        if self.canvascomponents['XYZ_Axis'] is not None:
            if self.canvasproperties['XYZ_Axis'] is True:
                self.canvascomponents['XYZ_Axis'].parent = None
                self.canvasproperties['XYZ_Axis'] = False
            else:
                self.canvascomponents['XYZ_Axis'].parent = self.view.scene
                self.canvasproperties['XYZ_Axis'] = True


    def clickBtnSnapshot(self):
        res = self.canvas.render()[:, :, 0:3]
        #
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getSaveFileName(None, 'Save canvas', self.settings['General']['RootPath'],
                                        filter="Portable Network Graphic file (PNG) (*.PNG);;All files (*.*)")
        if len(_file[0]) > 0:
            vispy_io.write_png(_file[0], res)


    def clickBtnGoHome(self):
        self.setCameraRange()
        self.view.camera.elevation = 30
        self.view.camera.azimuth = 135


    def changeCbbViewFrom(self):
        if self.cbbviewfrom.currentIndex() == 0:
            self.view.camera.elevation = 0
            self.view.camera.azimuth = 0
        if self.cbbviewfrom.currentIndex() == 1:
            self.view.camera.elevation = 0
            self.view.camera.azimuth = 90
        if self.cbbviewfrom.currentIndex() == 2:
            self.view.camera.elevation = 90
            self.view.camera.azimuth = 0


    def changeLdtZScale(self):
        #
        _zscale = basic_data.str2float(self.ldtzscale.text())
        if _zscale is not False and _zscale > 0.0:
            self.canvasproperties['Z_Scale'] = _zscale
        #
        # update seismic
        for _i in range(len(self.canvascomponents['Seismic'])):
            self.canvascomponents['Seismic'][_i].parent = None
            _vis = self.createVisualSeis(self.canvasproperties['Seismic'][_i]['Name'],
                                         self.canvasproperties['Seismic'][_i]['Orientation'],
                                         self.canvasproperties['Seismic'][_i]['Number'])
            if self.canvasproperties['Seismic'][_i]['Visible']:
                _vis.parent = self.view.scene
            self.canvascomponents['Seismic'][_i] = _vis
        #
        # update survey box
        _srvbox = self.createSrvBox()
        if len(_srvbox) == 12:
            # remove old one
            for _i in self.canvascomponents['Survey_Box']:
                _i.parent = None
            # plot new one
            self.canvascomponents['Survey_Box'] = _srvbox
            for _i in self.canvascomponents['Survey_Box']:
                _i.parent = self.view.scene
            self.canvasproperties['Survey_Box'] = True
        #
        # update xyz axis
        _xyzaxis = self.createXYZAxis()
        if _xyzaxis is not None:
            # remove old one
            self.canvascomponents['XYZ_Axis'].parent = None
            # plot new one
            self.canvascomponents['XYZ_Axis'] = _xyzaxis
            self.canvascomponents['XYZ_Axis'].parent = self.view.scene
            self.canvasproperties['XYZ_Axis'] = True
        #
        # set camara range
        self.setCameraRange()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def createVisualSeis(self, seis, ort, no):
        if self.checkSurvInfo() is False:
            return None
        if seis not in self.seisdata.keys():
            return None
        if ort != 'Inline' and ort != 'Crossline' and ort != 'Z':
            return None
        #
        _xlstart = self.survinfo['XLStart']
        _xlend = self.survinfo['XLEnd']
        _zstart = self.survinfo['ZStart'] * self.canvasproperties['Z_Scale']
        _zend = self.survinfo['ZEnd'] * self.canvasproperties['Z_Scale']
        _inlstart = self.survinfo['ILStart']
        _inlend = self.survinfo['ILEnd']
        _inlstep = self.survinfo['ILStep']
        _inlnum = self.survinfo['ILNum']
        if _inlnum == 1:
            _inlstep = 1
        _xlstep = self.survinfo['XLStep']
        _xlnum = self.survinfo['XLNum']
        if _xlnum == 1:
            _xlstep = 1
        _zstep = self.survinfo['ZStep'] * self.canvasproperties['Z_Scale']
        _znum = self.survinfo['ZNum']
        if _znum == 1:
            _zstep = 1
        #
        if ort == 'Z':
            no = no * self.canvasproperties['Z_Scale']
        #
        if no < _inlstart and no > _inlend \
            and no < _xlstart and no > _xlend \
            and no > _zstart and no < _zend:
            return None
        #
        _data = np.zeros([2, 2])
        _cmp = vis_cmap.makeColorMap(cmapname=self.seisvisconfig[seis]['Colormap'],
                                     flip=self.seisvisconfig[seis]['Flip'],
                                     opacity=self.seisvisconfig[seis]['Opacity'])
        _cmp = Colormap(_cmp.colors)
        _interp = self.seisvisconfig[seis]['Interpolation'].lower()
        if _interp is None or _interp == 'None' or _interp == 'none':
            _interp = 'nearest'
        _vis = scene.visuals.Image(_data, parent=None, cmap=_cmp,
                                   clim=(self.seisvisconfig[seis]['Minimum'], self.seisvisconfig[seis]['Maximum']),
                                   interpolation=_interp)
        _tr = scene.transforms.MatrixTransform()
        if ort == 'Inline':
            _idx = np.round((no - _inlstart) / _inlstep).astype(np.int32)
            _data = self.seisdata[seis][:, :, _idx]
            _tr.scale((abs(_xlstep), abs(_zstep)))
            _tr.rotate(-90, (1, 0, 0))
            _tr.translate((_xlstart, no, _zstart))
            _tr.translate((-0.5 * _xlstep, 0, -0.5 * _zstep))
        if ort == 'Crossline':
            _idx = np.round((no - _xlstart) / _xlstep).astype(np.int32)
            _data = self.seisdata[seis][:, _idx, :]
            _tr.scale((abs(_inlstep), abs(_zstep)))
            _tr.rotate(-90, (1, 0, 0))
            _tr.rotate(90, (0, 0, 1))
            _tr.translate((no, _inlstart, _zstart))
            _tr.translate((0, -0.5*_inlstep, -0.5*_zstep))
        if ort == 'Z':
            _idx = np.round((no - _zstart) / _zstep).astype(np.int32)
            _data = self.seisdata[seis][_idx, :, :]
            _tr.scale((abs(_inlstep), abs(_xlstep)))
            _tr.rotate(90, (0, 0, 1))
            _tr.rotate(180, (0, 1, 0))
            _tr.translate((_xlstart, _inlstart, no))
            _tr.translate((-0.5*_xlstep, -0.5*_inlstep, 0))
        _vis.set_data(_data)
        _vis.transform = _tr
        #
        return _vis


    def createSrvBox(self):
        _srvlines = []
        if self.checkSurvInfo():
            _xlstart = self.survinfo['XLStart']
            _xlend = self.survinfo['XLEnd']
            _zstart = self.survinfo['ZStart'] * self.canvasproperties['Z_Scale']
            _zend = self.survinfo['ZEnd'] * self.canvasproperties['Z_Scale']
            _inlstart = self.survinfo['ILStart']
            _inlend = self.survinfo['ILEnd']
            _inlstep = self.survinfo['ILStep']
            _inlnum = self.survinfo['ILNum']
            if _inlnum == 1:
                _inlstep = 1
            _xlstep = self.survinfo['XLStep']
            _xlnum = self.survinfo['XLNum']
            if _xlnum == 1:
                _xlstep = 1
            _zstep = self.survinfo['ZStep'] * self.canvasproperties['Z_Scale']
            _znum = self.survinfo['ZNum']
            if _znum == 1:
                _zstep = 1
            #
            for p in ([_xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep, _xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep, _xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep, _xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep, _xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep, _xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep, _xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep, _xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep, _xlend+0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zend+0.5*_zstep, _xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep, _xlstart-0.5*_xlstep, _inlend+0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zend+0.5*_zstep, _xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep],
                      [_xlstart-0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep, _xlend+0.5*_xlstep, _inlstart-0.5*_inlstep, _zstart-0.5*_zstep]):
                _line = scene.visuals.Line(pos=np.array([[p[0], p[1], p[2]], [p[3], p[4], p[5]]]),
                                           color='black', parent=None)
                _srvlines.append(_line)
        #
        return _srvlines


    def createXYZAxis(self):
        _xyz = None
        if self.checkSurvInfo():
            _xlstart = self.survinfo['XLStart']
            _xlend = self.survinfo['XLEnd']
            _zstart = self.survinfo['ZStart'] * self.canvasproperties['Z_Scale']
            _zend = self.survinfo['ZEnd'] * self.canvasproperties['Z_Scale']
            _inlstart = self.survinfo['ILStart']
            _inlend = self.survinfo['ILEnd']
            _inlstep = self.survinfo['ILStep']
            _inlnum = self.survinfo['ILNum']
            if _inlnum == 1:
                _inlstep = 1
            _xlstep = self.survinfo['XLStep']
            _xlnum = self.survinfo['XLNum']
            if _xlnum == 1:
                _xlstep = 1
            _zstep = self.survinfo['ZStep'] * self.canvasproperties['Z_Scale']
            _znum = self.survinfo['ZNum']
            if _znum == 1:
                _zstep = 1
            _x0 = 0.5 * (_xlstart + _xlend)  # _xlstart
            _y0 = 0.5 * (_inlstart + _inlend)  # _inlend + _inlend - _inlstart
            _z0 = 0.5 * (_zstart + _zend)  # _zend + _zend - _zstart
            _len = np.max(np.abs(np.array([_xlstep, _inlstep, _zstep])))
            _xyz = scene.visuals.XYZAxis(parent=None,
                                         pos=np.array([[_x0, _y0, _z0], [_x0 + _len, _y0, _z0],
                                                       [_x0, _y0, _z0], [_x0, _y0 + _len, _z0],
                                                       [_x0, _y0, _z0], [_x0, _y0, _z0 - _len]]))
        #
        return _xyz


    def setCameraRange(self):
        if self.checkSurvInfo():
            _xlstart = self.survinfo['XLStart']
            _xlend = self.survinfo['XLEnd']
            _zstart = self.survinfo['ZStart'] * self.canvasproperties['Z_Scale']
            _zend = self.survinfo['ZEnd'] * self.canvasproperties['Z_Scale']
            _inlstart = self.survinfo['ILStart']
            _inlend = self.survinfo['ILEnd']
            #
            self.view.camera.set_range((_xlstart, _xlend), (_inlstart, _inlend), (_zend, _zstart))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)



if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    PlotVisCanvas = QtWidgets.QWidget()
    gui = plotviscanvas()
    gui.setupGUI(PlotVisCanvas)
    PlotVisCanvas.show()
    sys.exit(app.exec_())


import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.encoding import ASCII, is_supported_encoding, UnsupportedEncodingError
from cognitivegeo.src.segpy.packer import HeaderPacker
from cognitivegeo.src.segpy.trace_header import TraceHeaderRev1
from cognitivegeo.src.segpy.toolkit import (write_textual_reel_header, write_binary_reel_header,
                                            write_trace_header, write_trace_samples,
                                            write_extended_textual_headers)



def write_segy(fh,
               seg_y_data,
               encoding=None,
               trace_header_format=TraceHeaderRev1,
               endian='>',
               progress=None):
    """
    Args:
        fh: A file-like object open for binary write.

        seg_y_data:  An object from which the headers and trace_samples data can be retrieved. Requires the following
            properties and methods:
              seg_y_data.textual_reel_header
              seg_y_data.binary_reel_header
              seg_y_data.extended_textual_header
              seg_y_data.trace_indexes
              seg_y_data.trace_header(trace_index)
              seg_y_data.trace_samples(trace_index)

              seg_y_data.encoding
              seg_y_data.endian

              One such legitimate object would be a SegYReader instance.

        trace_header_format: The class which defines the layout of the trace header. Defaults to TraceHeaderRev1.

        encoding: Optional encoding for text data. Typically 'cp037' for EBCDIC or 'ascii' for ASCII. If omitted, the
            seg_y_data object will be queries for an encoding property.

        endian: Big endian by default. If omitted, the seg_y_data object will be queried for an encoding property.

        progress: An optional progress bar object.

    Raises:
        UnsupportedEncodingError: If the specified encoding is neither ASCII nor EBCDIC
        UnicodeError: If textual data provided cannot be encoded into the required encoding.
    """

    encoding = encoding or (hasattr(seg_y_data, 'encoding') and seg_y_data.encoding) or ASCII

    if not is_supported_encoding(encoding):
        raise UnsupportedEncodingError("Writing SEG Y", encoding)

    write_textual_reel_header(fh, seg_y_data.textual_reel_header, encoding)
    write_binary_reel_header(fh, seg_y_data.binary_reel_header, endian)
    write_extended_textual_headers(fh, seg_y_data.extended_textual_header, encoding)

    trace_header_packer = HeaderPacker(trace_header_format, endian)

    for trace_index in seg_y_data.trace_indexes():
        write_trace_header(fh, seg_y_data.trace_header(trace_index), trace_header_packer)
        write_trace_samples(fh, seg_y_data.trace_samples(trace_index), seg_y_data.data_sample_format, endian=endian)



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# basic functions for processing image

import sys
import os
import numpy as np
import math
import matplotlib.pyplot as plt
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.colormap import colormap as vis_cmap


__all__ = ['image']

# List of all available interploation options
InterpolationList = ['None', 'Nearest', 'Quadric', 'Bilinear', 'Bicubic', 'Kaiser']


def plotImage(image, image_height, image_width, interpolation='bicubic',
              index_list=[], cmapname='red_white_blue', flipcmap=False):
    """
    Plot images (by matplotlib.imshow)

    Args:
        image:          2D matrix of images [pixel 1, pixel 2, ..., pixel N]
                        Each row for all pixels in an image
        image_height:   height of image
        image_width:    width of image
        interpolation:  Interpolation option. Default is 'bicubic'
        index_list:     index of the image row
        cmapname:       name of colormap as listed below:
                            'seismic', 'phase', 'frequency',
                            'red_white_blue',
                            'black_white_red', 'black_white_green', 'black_white_blue',
                            'white_gray_black',
                            'white_red_black', 'white_green_black', 'white_blue_black',
                            'black_red', 'black_green', 'black_blue'
                        Default is 'red_white_blue'
        flipcmap:   Flip colormap. Default is False

    Returns:
        None
    """

    if np.ndim(image) != 2:
        print('ERROR in plotImage: 2D image matrix expected')
        sys.exit()
    if image_height <= 1 or image_width <= 1:
        print('ERROR in plotImage: Image height/width be > 1')
        sys.exit()

    nimage, npixel = np.shape(image)

    if len(index_list) < 1 or np.min(index_list)<0 or np.max(index_list)>nimage-1:
        print('WARNING in plotImage: All images selected')
        index_list = np.linspace(0, nimage-1, nimage, dtype=np.int)

    index_list = index_list.astype(int)

    if npixel != image_height * image_width:
        print('ERROR in plotImage: Image height/width not match')
        sys.exit()
    for i in index_list:
        if i >=0 and i<nimage:
            plt.figure(facecolor='white')
            image_i = image[i, :]
            image_i = np.reshape(image_i, [image_height, image_width])
            plt.title('Image No. ' + str(i+1))
            plt.imshow(image_i, interpolation=interpolation,
                       cmap=vis_cmap.makeColorMap(cmapname, flipcmap))
            plt.axis('off')

    plt.show()

    return


def plotImageGallery(image, image_height, image_width, ncol=5,
                     cmapname='red_white_blue', flipcmap=False,
                     interpolation='bicubic',
                     spacing=False, maxfigsize=10,
                     valuemin=None, valuemax=None):
    """
    Plot image gallery (by matplotlib.imshow)

    Args:
        image:          2D matrix of images [nimage, npixel]
        image_height:   height of image
        image_width:    width of image
        ncol:           number of columns for subplotting. Default is 5
        cmapname:       name of colormap as listed below:
                            'seismic', 'phase', 'frequency',
                            'red_white_blue',
                            'black_white_red', 'black_white_green', 'black_white_blue',
                            'white_gray_black',
                            'white_red_black', 'white_green_black', 'white_blue_black',
                            'black_red', 'black_green', 'black_blue'
                        Default is 'red_white_blue'
        flipcmap:       Flip colormap. Default is False
        interpolation:  Interpolation option. Default is 'bicubic'
        maxfigsize:     Maximum figure size. Default is 10
        spacing:        Flag of spacing between images. Default is False
        valuemin:       Minimum value. Default is None (minimum of all images)
        valuemax:       Maximum value. Default is None (maximum of all images)

    Returns:
        None
    """

    if np.ndim(image) != 2:
        print('ERROR in plotImageGallery: 2D matrix expected for image')
        sys.exit()

    if ncol < 1:
        print('ERROR in plotImageGallery: Column number be >= 1')
        sys.exit()

    if maxfigsize <= 0:
        print('ERROR in plotImageGallery: Maximum figure size be >= 1')
        sys.exit()

    nimage = np.shape(image)[0]
    nrow = math.ceil(nimage / ncol)
    npixel = np.shape(image)[1]

    if npixel != image_height*image_width:
        print('ERROR in plotImageGallery: Image size not match')
        sys.exit()


    figwidth = float(ncol * image_width)
    figheight = float(nrow * image_height)
    if figwidth > figheight:
        figheight = figheight / figwidth * maxfigsize
        figwidth = maxfigsize
    else:
        figwidth = figwidth / figheight * maxfigsize
        figheight = maxfigsize

    plt.figure(facecolor='white', figsize=(figwidth, figheight))
    for i in range(nrow):
        for j in range(ncol):
            if i*ncol+j+1 > nimage:
                break
            if valuemax is None:
                _vmax = np.max(image[i * ncol + j, :])
            else:
                _vmax = valuemax
            if valuemin is None:
                _vmin = np.min(image[i * ncol + j, :])
            else:
                _vmin = valuemin
            plt.subplot(nrow, ncol, i * ncol + j + 1)
            plt.imshow(np.reshape(image[i * ncol + j, :], [image_height, image_width]),
                       interpolation=interpolation,
                       cmap=vis_cmap.makeColorMap(cmapname, flipcmap),
                       vmin=_vmin, vmax=_vmax,
                       )
            plt.axis('off')
    if spacing is False:
        plt.subplots_adjust(wspace=0.0, hspace=0.0)
    plt.show()

    return


class image:
    # Pack all functions as a class
    #
    InterpolationList = InterpolationList
    #
    plotImage = plotImage
    plotImageGallery = plotImageGallery


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     Movember 2018                                                                   #
#                                                                                           #
#############################################################################################

# Create a window for plot seismic traces


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.seismic.visualization import visualization as seis_vis
from cognitivegeo.src.gui.configlineplotting import configlineplotting as gui_configlineplotting
from cognitivegeo.src.gui.configplayer import configplayer as gui_configplayer
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class plotvis1dseisz(object):

    survinfo = {}
    seisdata = {}
    linestyle = core_set.Visual['Line']
    playerconfiginl = core_set.Viewer['Player']
    playerconfigxl = core_set.Viewer['Player']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    lineplottingconfig = {}


    def setupGUI(self, PlotVis1DSeisZ):
        PlotVis1DSeisZ.setObjectName("PlotVis1DSeisZ")
        PlotVis1DSeisZ.setFixedSize(500, 370)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/waveform.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVis1DSeisZ.setWindowIcon(icon)
        #
        self.lwgattrib = QtWidgets.QListWidget(PlotVis1DSeisZ)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(10, 10, 230, 290))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        #
        self.lblinl = QtWidgets.QLabel(PlotVis1DSeisZ)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(260, 50, 150, 30))
        self.slbinl = QtWidgets.QScrollBar(PlotVis1DSeisZ)
        self.slbinl.setObjectName("slbinl")
        self.slbinl.setOrientation(QtCore.Qt.Horizontal)
        self.slbinl.setGeometry(QtCore.QRect(260, 80, 170, 30))
        self.ldtinl = QtWidgets.QLineEdit(PlotVis1DSeisZ)
        self.ldtinl.setObjectName("ldtinl")
        self.ldtinl.setGeometry(QtCore.QRect(440, 80, 50, 30))
        self.ldtinl.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.lblxl = QtWidgets.QLabel(PlotVis1DSeisZ)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(260, 120, 150, 30))
        self.slbxl = QtWidgets.QScrollBar(PlotVis1DSeisZ)
        self.slbxl.setObjectName("slbxl")
        self.slbxl.setOrientation(QtCore.Qt.Horizontal)
        self.slbxl.setGeometry(QtCore.QRect(260, 150, 170, 30))
        self.ldtxl = QtWidgets.QLineEdit(PlotVis1DSeisZ)
        self.ldtxl.setObjectName("ldtxl")
        self.ldtxl.setGeometry(QtCore.QRect(440, 150, 50, 30))
        self.ldtxl.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.lblrange = QtWidgets.QLabel(PlotVis1DSeisZ)
        self.lblrange.setObjectName("lblrange")
        self.lblrange.setGeometry(QtCore.QRect(260, 200, 40, 30))
        self.ldtmin = QtWidgets.QLineEdit(PlotVis1DSeisZ)
        self.ldtmin.setObjectName("ldtmin")
        self.ldtmin.setGeometry(QtCore.QRect(300, 200, 70, 30))
        self.ldtmin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtmax = QtWidgets.QLineEdit(PlotVis1DSeisZ)
        self.ldtmax.setObjectName("ldtmax")
        self.ldtmax.setGeometry(QtCore.QRect(420, 200, 70, 30))
        self.ldtmax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblrangeto = QtWidgets.QLabel(PlotVis1DSeisZ)
        self.lblrangeto.setObjectName("lblrangeto")
        self.lblrangeto.setGeometry(QtCore.QRect(370, 200, 50, 30))
        self.lblrangeto.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.btnconfigline = QtWidgets.QPushButton(PlotVis1DSeisZ)
        self.btnconfigline.setObjectName("btnconfigline")
        self.btnconfigline.setGeometry(QtCore.QRect(330, 270, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/plotcurve.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigline.setIcon(icon)
        #
        self.btnconfigplayer = QtWidgets.QPushButton(PlotVis1DSeisZ)
        self.btnconfigplayer.setObjectName("btnconfigplayer")
        self.btnconfigplayer.setGeometry(QtCore.QRect(370, 10, 120, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/video.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigplayer.setIcon(icon)
        #
        self.btnplot = QtWidgets.QPushButton(PlotVis1DSeisZ)
        self.btnplot.setObjectName("btnplot")
        self.btnplot.setGeometry(QtCore.QRect(170, 320, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/waveform.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplot.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVis1DSeisZ)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVis1DSeisZ.geometry().center().x()
        _center_y = PlotVis1DSeisZ.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVis1DSeisZ)
        QtCore.QMetaObject.connectSlotsByName(PlotVis1DSeisZ)



    def retranslateGUI(self, PlotVis1DSeisZ):
        self.dialog = PlotVis1DSeisZ
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVis1DSeisZ.setWindowTitle(_translate("PlotVis1DSeisZ", "1D Window: Seismic Waveform"))
        if self.checkSurvInfo() is True:
            _firstattrib = None
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_translate("PlotVis1DSeisZ", i))
                    self.lwgattrib.addItem(item)
                    if _firstattrib is None:
                        _firstattrib = item
            self.lwgattrib.setCurrentItem(_firstattrib)
        self.lwgattrib.itemSelectionChanged.connect(self.changeLwgAttrib)
        #
        self.lblinl.setText(_translate("PlotVis1DSeisZ", "@ Inline:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['ILRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbinl.setMinimum(_slicemin)
        self.slbinl.setMaximum(_slicemax)
        self.slbinl.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtinl.setText(_translate("PlotVis1DSeisZ", str(_slicemin)))
        else:
            self.ldtinl.setText(_translate("PlotVis1DSeisZ", ''))
        self.slbinl.valueChanged.connect(self.changeSlbInl)
        self.ldtinl.textChanged.connect(self.changeLdtInl)
        #
        self.lblxl.setText(_translate("PlotVis1DSeisZ", "@ crossline:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbxl.setMinimum(_slicemin)
        self.slbxl.setMaximum(_slicemax)
        self.slbxl.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtxl.setText(_translate("PlotVis1DSeisZ", str(_slicemin)))
        else:
            self.ldtxl.setText(_translate("PlotVis1DSeisZ", ''))
        self.slbxl.valueChanged.connect(self.changeSlbXl)
        self.ldtxl.textChanged.connect(self.changeLdtXl)
        #
        self.lblrange.setText(_translate("PlotVis1DSeisZ", "Range:"))
        self.lblrangeto.setText(_translate("PlotVis1DSeisZ", "~~~"))
        if (self.checkSurvInfo() is True) \
                and (self.lwgattrib.currentItem() is not None) \
                and (self.checkSeisData(self.lwgattrib.currentItem().text()) is True):
            _min, _max = self.getAttribRange(self.lwgattrib.currentItem().text())
            self.ldtmin.setText(_translate("PlotVis1DSeisZ", str(_min)))
            self.ldtmax.setText(_translate("PlotVis1DSeisZ", str(_max)))
        #
        self.btnconfigline.setText(_translate("PlotVis1DSeisZ", "Waveform Configuration"))
        self.btnconfigline.clicked.connect(self.clickBtnConfigLine)
        if (self.checkSurvInfo() is True) \
                and (self.lwgattrib.currentItem() is not None) \
                and (self.checkSeisData(self.lwgattrib.currentItem().text()) is True):
            _config = self.linestyle
            self.lineplottingconfig[self.lwgattrib.currentItem().text()] = _config
        #
        self.btnconfigplayer.setText(_translate("PlotVis1DSeisZ", "Waveform Player"))
        self.btnconfigplayer.clicked.connect(self.clickBtnConfigPlayer)
        #
        self.btnplot.setText(_translate("PlotVis1DSeisZ", "Seismic Waveform Viewer"))
        self.btnplot.setDefault(True)
        self.btnplot.clicked.connect(self.clickBtnPlot)


    def changeLwgAttrib(self):
        if len(self.lwgattrib.selectedItems()) > 0:
            # if len(self.lwgattrib.selectedItems()) > 1:
            #     self.ldtmin.setEnabled(False)
            #     self.ldtmax.setEnabled(False)
            # else:
            #     self.ldtmin.setEnabled(True)
            #     self.ldtmax.setEnabled(True)
            _slices = self.survinfo['ILRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbinl.setMinimum(_slicemin)
            self.slbinl.setMaximum(_slicemax)
            self.ldtinl.setText(str(self.slbinl.value()))
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbxl.setMinimum(_slicemin)
            self.slbxl.setMaximum(_slicemax)
            self.ldtxl.setText(str(self.slbxl.value()))
            _min, _max = self.getAttribRange(self.lwgattrib.selectedItems()[0].text())
            self.ldtmin.setText(str(_min))
            self.ldtmax.setText(str(_max))
            #
            _config = {}
            for _attrib in self.lwgattrib.selectedItems():
                if _attrib.text() in self.lineplottingconfig.keys():
                    _config[_attrib.text()] = self.lineplottingconfig[_attrib.text()]
                else:
                    _config[_attrib.text()] = self.linestyle
            self.lineplottingconfig = _config
        else:
            self.slbinl.setMinimum(0)
            self.slbinl.setMaximum(0)
            self.ldtinl.setText('')
            self.slbxl.setMinimum(0)
            self.slbxl.setMaximum(0)
            self.ldtxl.setText('')
            self.ldtmin.setText('')
            self.ldtmax.setText('')
            self.lineplottingconfig = {}


    def changeSlbInl(self):
        self.ldtinl.setText(str(self.slbinl.value()))


    def changeLdtInl(self):
        if len(self.ldtinl.text()) > 0:
            _val = basic_data.str2int(self.ldtinl.text())
            if _val >= self.slbinl.minimum() and _val <= self.slbinl.maximum():
                self.slbinl.setValue(_val)


    def changeSlbXl(self):
        self.ldtxl.setText(str(self.slbxl.value()))


    def changeLdtXl(self):
        if len(self.ldtxl.text()) > 0:
            _val = basic_data.str2int(self.ldtxl.text())
            if _val >= self.slbxl.minimum() and _val <= self.slbxl.maximum():
                self.slbxl.setValue(_val)


    def clickBtnConfigLine(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configlineplotting()
        _gui.lineplottingconfig = self.lineplottingconfig
        _gui.setupGUI(_config)
        _config.exec()
        self.lineplottingconfig = _gui.lineplottingconfig
        _config.show()


    def clickBtnConfigPlayer(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configplayer()
        _gui.playerconfig = {}
        _gui.playerconfig['Inline'] = self.playerconfiginl
        _gui.playerconfig['Crossline'] = self.playerconfigxl
        _gui.setupGUI(_config)
        _config.exec()
        self.playerconfiginl = _gui.playerconfig['Inline']
        self.playerconfigxl = _gui.playerconfig['Crossline']
        _config.show()


    def clickBtnPlot(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in PlotVis1DSeisZ: No property selected for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '1D Window: Seismic Waveform',
                                           'No property selected for plot')
            return
        #
        _inls = self.slbinl.value()
        _xls = self.slbxl.value()
        _min = basic_data.str2float(self.ldtmin.text())
        _max = basic_data.str2float(self.ldtmax.text())
        if _min is False or _max is False:
            vis_msg.print("ERROR in PlotVis1DSeisZ: Non-float range specified for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '1D Window: Seismic Waveform',
                                           'Non-float range specified for plot')
            return

        for i in range(len(_attriblist)):
            print("PlotVis1DSeisZ: Plot %d of %d properties: %s" % (i + 1, len(_attriblist), _attriblist[i].text()))
            _data = self.seisdata[_attriblist[i].text()]
            # if len(_attriblist) > 1:
            #     _min, _max = self.getAttribRange(_attriblist[i].text())
            #
            _color = self.lineplottingconfig[_attriblist[i].text()]['Color'].lower()
            _linewidth = self.lineplottingconfig[_attriblist[i].text()]['Width']
            _linestyle = self.lineplottingconfig[_attriblist[i].text()]['Style'].lower()
            _markerstyle = self.lineplottingconfig[_attriblist[i].text()]['MarkerStyle']
            _markersize = self.lineplottingconfig[_attriblist[i].text()]['MarkerSize']
            #
            seis_vis.plotSeisZTracePlayerFrom3DMat(_data, initinltc=_inls, initxltc=_xls, seisinfo=self.survinfo,
                                                   titlesurf=_attriblist[i].text() + ' at ',
                                                   valuemin=_min, valuemax=_max,
                                                   color=_color, markerstyle=_markerstyle, markersize=_markersize,
                                                   linewidth=_linewidth, linestyle=_linestyle,
                                                   playerconfiginl=self.playerconfiginl,
                                                   playerconfigxl=self.playerconfigxl,
                                                   fontstyle=self.fontstyle,
                                                   qicon=QtGui.QIcon(os.path.join(self.iconpath, "icons/logo.png")))
        #
        # QtWidgets.QMessageBox.information(self.msgbox,
        #                                   "Plot Inline",
        #                                   str(len(_attriblist)) + " properties plotted successfully")
        self.dialog.close()
        return


    def getAttribRange(self, f):
        _min = -1
        _max = 1
        if (self.checkSurvInfo() is True) \
                and (f in self.seisdata.keys()) \
                and (self.checkSeisData(f) is True):
            _min = np.min(self.seisdata[f])
            _max = np.max(self.seisdata[f])
        return _min, _max


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("PlotVis1DSeisZ: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Plot Seismic Waveform',
            #                                'Survey not found')
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    PlotVis1DSeisZ = QtWidgets.QWidget()
    gui = plotvis1dseisz()
    gui.setupGUI(PlotVis1DSeisZ)
    PlotVis1DSeisZ.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# basic functions for processing image

import sys, os
import numpy as np
from scipy import interpolate
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg

__all__ = ['image']


def changeImageSize(image, image_height, image_width,
                    image_height_new, image_width_new,
                    kind='cubic'):
    """
    Change image size through 2D interpolation (from scipy)

    Args:
        image:              2D matrix of images [pixel 1, pixel 2, ..., pixel N]
                            Each row for all pixels in an image
        image_height:       Original height of image
        image_width:        oOriginal width of image
        image_height_new:   New height of image
        image_width_new:    New width of image
        kind:               Interpolation type 'linear', 'cubic', or 'quintic'. Default is 'cubic

    Returns:
        2D matrix of images after interpolation, with each row representing one image
    """

    if np.ndim(image) != 2:
        vis_msg.print('ERROR in changeImageSize: 2D image matrix expected', type='error')
        sys.exit()
    if image_height <= 1 or image_width <= 1:
        vis_msg.print('ERROR in changeImageSize: Original image height/width be > 1', type='error')
        sys.exit()
    if image_height_new <= 0 or image_width_new <= 0:
        vis_msg.print('ERROR in changeImageSize: New image height/width be >= 1', type='error')
        sys.exit()

    nimage, npixel = np.shape(image)

    if npixel != image_height * image_width :
        vis_msg.print('ERROR in changeImageSize: Original image height/width not match', type='error')
        sys.exit()


    height = np.linspace(0.0, 1.0, image_height)
    width = np.linspace(0.0, 1.0, image_width)
    height_new = np.linspace(0.0, 1.0, image_height_new)
    width_new = np.linspace(0.0, 1.0, image_width_new)
    npixel_new = image_height_new * image_width_new

    image_new = np.zeros([nimage, npixel_new])
    for i in range(nimage):
        image_i = image[i, :]
        image_i = np.reshape(image_i, [image_height, image_width])
        f = interpolate.interp2d(width, height, image_i, kind=kind)
        image_i_new = f(width_new, height_new)
        image_new[i, :] = np.reshape(image_i_new, [1, npixel_new])

    return image_new


def rotateImage(image, image_height, image_width, flag='180'):
    """
    Rotate images

    Args:
        image:          2D matrix of images [pixel 1, pixel 2, ..., pixel N]
                        Each row for all pixels in an image
        image_height:   height of image
        image_width:    width of image
        flag:           type of rotation. Default is '180'
                        180 for 180-degree rotation (anticlockwise)
                        90 for 90-degree rotation
                        270 for 270-degree rotation
                        lr for horizontally flipping
                        ud for vertically flipping

    Returns:
        2D matrix of images after rotation, with each row representing one image
    """

    if np.ndim(image) != 2:
        vis_msg.print('ERROR in rotateImage: 2D image matrix expected', type='error')
        sys.exit()
    # if image_height <= 1 or image_width <= 1:
    #     print('ERROR in rotateImage: Original image height/width be > 1')
    #     sys.exit()
    nimage, npixel = np.shape(image)
    if npixel != image_height * image_width:
        vis_msg.print('ERROR in rotateImage: Image height/width not match', type='error')
        sys.exit()
    #
    image_rotate = np.reshape(image, [nimage, image_height, image_width])
    #
    if flag == '90':
        image_rotate = np.flip(np.transpose(image_rotate, [0, 2, 1]), axis=1)
    if flag == '180':
        image_rotate = np.flip(np.flip(image_rotate, axis=1), axis=2)
    if flag == '270':
        image_rotate = np.flip(np.transpose(image_rotate, [0, 2, 1]), axis=2)
    if flag == 'lr':
        image_rotate = np.flip(image_rotate, axis=2)
    if flag == 'ud':
        image_rotate = np.flip(image_rotate, axis=1)
    #
    return np.reshape(image_rotate, [nimage, npixel])


class image:
    # Pack all functions as a class
    #
    changeImageSize = changeImageSize
    rotateImage = rotateImage


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# pre-stack seismic data IO
from PyQt5 import QtCore
import os
import sys
import numpy as np
import numpy.matlib as npmat
import math
#
sys.path.append(os.path.dirname(__file__)[:-10][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg
import cognitivegeo.src.segpy.reader as syreader
import cognitivegeo.src.segpy.packer as sypacker
from cognitivegeo.src.segpy.trace_header import TraceHeaderRev1
from cognitivegeo.src.segpy.header import field
from cognitivegeo.src.segpy.types import Int32

__all__ = ['inputoutput']


def scanSegy(segyfile, traceheaderformat=TraceHeaderRev1, endian='>', verbose=True):
    """
    Scan a given segy file

    Args:
        segyfile:           Input segy file as a string
        traceheaderformat:  Format of trace header (as defined in segpy). Default is TraceHeaderRev1
        endian:             Endian option ('>' or '<'). Default is '>'
        verbose:            Verbose option of scanning progress (True of False). Default is True

    Return:
        all_shots:          All shots of the input segy file as a dictionary, with the record number as keys.
                            Each shot is representing as a dictionary, with the following keys,
                                'Data' ----- A list of all traces
                                'Trace_List' ----- List of the available trace number
        num_samples:        Number of samples in each trace
        sample_interval:    Sample interval in ms.
    """


    traceheaderpacker = sypacker.HeaderPacker(traceheaderformat, endian)
    dataformat = np.array([4, 4, 2, 4, 4, 0, 0, 1])
    with open(segyfile, 'rb') as sid:
        dat = syreader.create_reader(sid, trace_header_format=traceheaderformat, endian=endian)
        binary_header = syreader.read_binary_reel_header(sid, endian=endian)
        num_samples = binary_header.num_samples
        sample_interval = binary_header.sample_interval / 1000
        databyte = dataformat[binary_header.data_sample_format - 1]
        #
        all_shots = {}
        pos = np.int64(3200 + 400)
        for itrace in dat.trace_indexes():
            if verbose:
                sys.stdout.write(
                    '\r>>> Scan %s: %d traces ' % (segyfile, itrace + 1))
                sys.stdout.flush()
            #
            header = syreader.read_trace_header(sid, trace_header_packer=traceheaderpacker, pos=pos)
            curr_ident = header.field_record_num
            #
            trace_data = dat.trace_samples(itrace)
            if curr_ident not in all_shots.keys():
                all_shots[curr_ident] = {}
                all_shots[curr_ident]['Trace_List'] = [header.trace_num]
                all_shots[curr_ident]['Data'] = {}
                all_shots[curr_ident]['Data']['Current_Trace'] = 0
                all_shots[curr_ident]['Data'][0] = trace_data
            else:
                all_shots[curr_ident]['Trace_List'].append(header.trace_num)
                # all_shots[curr_ident]['Data'] = \
                #     np.concatenate((all_shots[curr_ident]['Data'], trace_data), axis=1)
                # alternative approch of using np.concatenate that is slow
                all_shots[curr_ident]['Data']['Current_Trace'] += 1
                curr_trace = all_shots[curr_ident]['Data']['Current_Trace']
                all_shots[curr_ident]['Data'][curr_trace] = trace_data
            pos = pos + 240 + header.num_samples * databyte
        if verbose:
            print('Done')
    #
    return all_shots, num_samples, sample_interval



def readPsSeisFromSegy(segyfile, traceheaderformat=TraceHeaderRev1, endian='>',
                       verbose=True, qpgsdlg=None):
    """
    Read pre-stack seismic from segy

    Args:
        segy:               segy file
        traceheaderformat:  Format of trace header (as defined in segpy). Default is TraceHeaderRev1
        endian:             Endian option ('>' or '<'). Default is '>'
        verbose:            Verbose option of scanning progress (True of False). Default is True
        qpgsdlg:            Progress dialog window. Default is None

    Return:
         pre-stack seismic a dictionary of all gathers, with each gather as a dictionary of two keys:
                ShotData:   gather data as a 3D mat
                ShotInfo:   gather information dictionary as defined in analysis.py
    """

    if os.path.exists(segyfile) is False or os.path.isfile(segyfile) is False:
        vis_msg.print("ERROR in readSeisPSFromSegy: No pre-stack segy found", type='error')
        sys.exit()
    #
    all_shots, num_samples, sample_interval = \
        scanSegy(segyfile, traceheaderformat=traceheaderformat, endian=endian)
    #
    if verbose is True:
        print("Read " + segyfile + ": " + str(len(all_shots)) + " shots")
    #
    psseis = {}
    #
    if qpgsdlg is not None:
        qpgsdlg.setMaximum(len(all_shots))
    #
    ishot = 0
    for shot in all_shots:
        #
        trace_list = np.unique(all_shots[shot]['Trace_List'])
        trace_step = 1
        if len(trace_list) > 1:
            trace_step = min(np.abs(trace_list[0:-1]-trace_list[1:]))
        trace_start = np.min(trace_list)
        trace_num_per_line = int((np.max(trace_list) - np.min(trace_list)) / trace_step) + 1
        trace_end = trace_start + (trace_num_per_line-1)*trace_step
        #
        trace_id_previous = trace_end + 1
        nline = 0
        for itrace in range(len(all_shots[shot]['Trace_List'])):
            trace_id = all_shots[shot]['Trace_List'][itrace]
            # check if a new line
            if trace_id <= trace_id_previous:
                nline += 1
            trace_id_previous = trace_id
        #
        if qpgsdlg is not None:
            QtCore.QCoreApplication.instance().processEvents()
            qpgsdlg.setValue(ishot)
        #
        if verbose:
            sys.stdout.write(
                '\r>>> Read %d of %d shots: %s, %d line x %d traces/line (from %d to %d with step %d)'
                % (ishot+1, len(all_shots), str(shot), nline, trace_num_per_line, trace_start, trace_end, trace_step))
            sys.stdout.flush()
        ishot += 1
        # print('>>> Shot %s contains %d lines with %d traces per line (from %d to %d with step %d)'
        #       %(str(shot), nline, trace_num_per_line, trace_start, trace_end, trace_step))
        #
        psdata = {}
        psdata['ShotInfo'] = {}
        psdata['ShotInfo']['ZNum'] = num_samples
        psdata['ShotInfo']['ILNum'] = nline
        psdata['ShotInfo']['XLNum'] = trace_num_per_line
        #
        psdata['ShotInfo']['ZStart'] = 0
        psdata['ShotInfo']['ZStep'] = - sample_interval
        psdata['ShotInfo']['ZEnd'] = 0 + (psdata['ShotInfo']['ZNum'] - 1) * psdata['ShotInfo']['ZStep']
        psdata['ShotInfo']['ZRange'] = np.linspace(psdata['ShotInfo']['ZStart'], psdata['ShotInfo']['ZEnd'],
                                                   psdata['ShotInfo']['ZNum'])
        psdata['ShotInfo']['XLStart'] = trace_start
        psdata['ShotInfo']['XLStep'] = trace_step
        psdata['ShotInfo']['XLEnd'] = trace_end
        psdata['ShotInfo']['XLRange'] = np.linspace(trace_start, trace_end, trace_num_per_line)
        psdata['ShotInfo']['ILStart'] = 0
        psdata['ShotInfo']['ILStep'] = 1
        psdata['ShotInfo']['ILEnd'] = nline - 1
        psdata['ShotInfo']['ILRange'] = np.linspace(0, nline-1, nline)
        #
        psdata['ShotData'] = np.zeros([num_samples, trace_num_per_line, nline])
        psdata['ShotInfo']['TraceFlag'] = np.ones([trace_num_per_line, nline])
        #
        trace_id_previous = trace_end + 1
        idx_line = -1
        for itrace in range(len(all_shots[shot]['Trace_List'])):
            trace_id = all_shots[shot]['Trace_List'][itrace]
            # check if a new line
            if trace_id <= trace_id_previous:
                idx_line = idx_line + 1
            trace_id_previous = trace_id
            #
            idx_trace = int((trace_id - trace_start) / trace_step)
            psdata['ShotData'][:, idx_trace, idx_line] = all_shots[shot]['Data'][itrace]
            psdata['ShotInfo']['TraceFlag'][idx_trace, idx_line] = 0
        #
        psseis[str(shot)] = psdata
    #
    if qpgsdlg is not None:
        qpgsdlg.setValue(len(all_shots))
    if verbose:
        print('Done')

    return psseis


def defSegyTraceHeaderFormat(record_byte=9, trace_byte=13):
    """
    Define a TraceHeaderFormat with given keywords

    Args:
        record_byte:    Byte of the field record number in trace header. Default is 9
        trace_byte:     Byte of the trace number in trace header. Default is 13

    Return:
        SegyTraceHeaderFormat as defined in segpy
    """

    class SegyTraceHeaderFormat(TraceHeaderRev1):
        field_record_num = field(
            Int32, offset=record_byte, default=0, documentation=
            "Original field record number. Highly recommended for all types of data."
        )

        trace_num = field(
            Int32, offset=trace_byte, default=0, documentation=
            "Trace number within the original field record. Highly recommended for all types of data."
        )

    return SegyTraceHeaderFormat


class inputoutput:
    # pack all functions as a class
    #
    readPsSeisFromSegy = readPsSeisFromSegy
    #
    defSegyTraceHeaderFormat = defSegyTraceHeaderFormat


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     August 2018                                                                     #
#                                                                                           #
#############################################################################################

# Create a window for converting point sets to seismic


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.pointset.analysis import analysis as point_ays
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class convertpointset2seis(object):

    survinfo = {}
    seisdata = {}
    pointsetdata = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ConvertPointSet2Seis):
        ConvertPointSet2Seis.setObjectName("ConvertPointSet2Seis")
        ConvertPointSet2Seis.setFixedSize(400, 410)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/point.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConvertPointSet2Seis.setWindowIcon(icon)
        #
        self.lblpoint = QtWidgets.QLabel(ConvertPointSet2Seis)
        self.lblpoint.setObjectName("lblpoint")
        self.lblpoint.setGeometry(QtCore.QRect(10, 10, 170, 30))
        self.lwgpoint = QtWidgets.QListWidget(ConvertPointSet2Seis)
        self.lwgpoint.setObjectName("lwgpoint")
        self.lwgpoint.setGeometry(QtCore.QRect(10, 50, 170, 200))
        self.lwgpoint.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.lblarrow = QtWidgets.QLabel(ConvertPointSet2Seis)
        self.lblarrow.setObjectName("lblarrow")
        self.lblarrow.setGeometry(QtCore.QRect(180, 110, 40, 30))
        self.lblattrib = QtWidgets.QLabel(ConvertPointSet2Seis)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(220, 10, 170, 30))
        self.lwgattrib = QtWidgets.QListWidget(ConvertPointSet2Seis)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(220, 50, 170, 200))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.lblvalue = QtWidgets.QLabel(ConvertPointSet2Seis)
        self.lblvalue.setObjectName("lblvalue")
        self.lblvalue.setGeometry(QtCore.QRect(220, 270, 80, 30))
        self.ldtvalue = QtWidgets.QLineEdit(ConvertPointSet2Seis)
        self.ldtvalue.setObjectName("ldtvalue")
        self.ldtvalue.setGeometry(QtCore.QRect(310, 270, 80, 30))
        self.lbloverlap = QtWidgets.QLabel(ConvertPointSet2Seis)
        self.lbloverlap.setObjectName("lbloverlap")
        self.lbloverlap.setGeometry(QtCore.QRect(220, 310, 80, 30))
        self.cbboverlap = QtWidgets.QComboBox(ConvertPointSet2Seis)
        self.cbboverlap.setObjectName("cbboverlap")
        self.cbboverlap.setGeometry(QtCore.QRect(310, 310, 80, 30))
        self.btnapply = QtWidgets.QPushButton(ConvertPointSet2Seis)
        self.btnapply.setObjectName("btnedit")
        self.btnapply.setGeometry(QtCore.QRect(150, 360, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConvertPointSet2Seis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConvertPointSet2Seis.geometry().center().x()
        _center_y = ConvertPointSet2Seis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConvertPointSet2Seis)
        QtCore.QMetaObject.connectSlotsByName(ConvertPointSet2Seis)


    def retranslateGUI(self, ConvertPointSet2Seis):
        self.dialog = ConvertPointSet2Seis
        #
        _translate = QtCore.QCoreApplication.translate
        ConvertPointSet2Seis.setWindowTitle(_translate("ConvertPointSet2Seis", "Convert PointSet to Seismic"))
        self.lblpoint.setText(_translate("ConvertPointSet2Seis", "Select pointsets:"))
        self.lwgpoint.itemSelectionChanged.connect(self.changeLwgPoint)
        self.lblarrow.setText(_translate("ConvertPointSet2Seis", "==>"))
        self.lblarrow.setAlignment(QtCore.Qt.AlignCenter)
        self.lblattrib.setText(_translate("ConvertPointSet2Seis", "Select properties:"))
        self.lblvalue.setText(_translate("ConvertPointSet2Seis", "Undefined value:"))
        self.ldtvalue.setText(_translate("ConvertPointSet2Seis", "-999"))
        self.lbloverlap.setText(_translate("ConvertPointSet2Seis", "Overlap points:"))
        self.cbboverlap.addItems(['Sum', 'Average', 'Maximum', 'Minimum'])
        self.btnapply.setText(_translate("ConvertPointSet2Seis", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)
        #
        self.lwgpoint.clear()
        if len(self.pointsetdata.keys()) > 0:
            for i in sorted(self.pointsetdata.keys()):
                if self.checkPointSet(i):
                    item = QtWidgets.QListWidgetItem(self.lwgpoint)
                    item.setText(i)
                    self.lwgpoint.addItem(item)


    def changeLwgPoint(self):
        self.lwgattrib.clear()
        _firstattrib = None
        #
        _pointlist = self.lwgpoint.selectedItems()
        _pointlist = [f.text() for f in _pointlist]
        if len(_pointlist) > 0:
            for _k in self.pointsetdata[_pointlist[0]].keys():
                _flag = True
                for _i in range(len(_pointlist)):
                    if _k not in self.pointsetdata[_pointlist[_i]].keys():
                        _flag = False
                        break
                if _flag is True:
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_k)
                    self.lwgpoint.addItem(item)
                    if _firstattrib is None:
                        _firstattrib = item
        if _firstattrib is not None:
            self.lwgattrib.setCurrentItem(_firstattrib)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        _pointlist = self.lwgpoint.selectedItems()
        _pointlist = [f.text() for f in _pointlist]
        if len(_pointlist) < 1:
            vis_msg.print("ERROR in ConvertPointSet2Seis: No pointset selected for conversion", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert PointSet to Seismic',
                                           'No pointset selected for conversion')
            return
        #
        _attriblist = self.lwgattrib.selectedItems()
        _attriblist = [f.text() for f in _attriblist]
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in ConvertPointSet2Seis: No property selected for conversion", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                            'Convert PointSet to Seismic',
                                            'No property selected for conversion')
            return
        #
        if basic_data.str2float(self.ldtvalue.text()) is False:
            vis_msg.print("ERROR in ConvertPointSet2Seis: Non-float undefined value", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert PointSet to Seismic',
                                           'Non-float undefined value')
            return
        #
        if checkSurvInfo(self.survinfo) is False:
            vis_msg.print("ERROR in ConvertPointSet2Seis: No seismic survey found for conversion", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert PointSet to Seismic',
                                           'No seismic survey found for conversion')
            return
        #
        for _f in _attriblist:
            if _f in self.seisdata.keys() and checkSeisData(self.seisdata[_f], self.survinfo):
                reply = QtWidgets.QMessageBox.question(self.msgbox, 'Convert PointSet to Seismic',
                                                       _f + ' already exists. Overwrite?',
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return
        #
        _survinfo = self.survinfo
        for _f in _attriblist:
            _seisdata = np.zeros([_survinfo['ZNum'], _survinfo['XLNum'], _survinfo['ILNum']]) + float(self.ldtvalue.text())
            for _p in _pointlist:
                _pointsetdata = basic_mdt.exportMatDict(self.pointsetdata[_p], ['Inline', 'Crossline', 'Z', _f])
                _pointsetdata = seis_ays.removeOutofSurveySample(_pointsetdata,
                                                              inlstart=_survinfo['ILStart'], inlend=_survinfo['ILEnd'],
                                                              xlstart=_survinfo['XLStart'], xlend=_survinfo['XLEnd'],
                                                              zstart=_survinfo['ZStart'], zend=_survinfo['ZEnd'])
                if np.shape(_pointsetdata)[0] < 1:
                    continue
                _ijk = seis_ays.convertIXZToIJK(_pointsetdata,
                                                inlstart=_survinfo['ILStart'], inlstep=_survinfo['ILStep'],
                                                xlstart=_survinfo['XLStart'], xlstep=_survinfo['XLStep'],
                                                zstart=_survinfo['ZStart'], zstep=_survinfo['ZStep'])
                for _i in range(np.shape(_ijk)[0]):
                    _ijk_i = int(_ijk[_i, 0])
                    _ijk_j = int(_ijk[_i, 1])
                    _ijk_k = int(_ijk[_i, 2])
                    _data = _seisdata[_ijk_k, _ijk_j, _ijk_i]
                    if float(_data) == float(self.ldtvalue.text()):
                        _seisdata[_ijk_k, _ijk_j, _ijk_i] = _pointsetdata[_i, 3]
                    else:
                        if self.cbboverlap.currentIndex() == 0:
                            _seisdata[_ijk_k, _ijk_j, _ijk_i] = _pointsetdata[_i, 3] + _data
                        if self.cbboverlap.currentIndex() == 1:
                            _seisdata[_ijk_k, _ijk_j, _ijk_i] = 0.5 * (_pointsetdata[_i, 3] + _data)
                        if self.cbboverlap.currentIndex() == 2:
                            _seisdata[_ijk_k, _ijk_j, _ijk_i] = np.max([_pointsetdata[_i, 3], _data])
                        if self.cbboverlap.currentIndex() == 3:
                            _seisdata[_ijk_k, _ijk_j, _ijk_i] = np.min([_pointsetdata[_i, 3], _data])

            self.seisdata[_f] = _seisdata
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Convert PointSet to Seismic",
                                          "PointSet converted successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkPointSet(self, name):
        return point_ays.checkPoint(self.pointsetdata[name])

def checkSurvInfo(survinfo):
    return seis_ays.checkSeisInfo(survinfo)

def checkSeisData(seisdata, survinfo):
    return seis_ays.isSeis3DMatConsistentWithSeisInfo(seisdata, survinfo)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConvertPointSet2Seis = QtWidgets.QWidget()
    gui = convertpointset2seis()
    gui.setupGUI(ConvertPointSet2Seis)
    ConvertPointSet2Seis.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

from __future__ import absolute_import, division, print_function

__all__ = ['io', 'ays', 'vis']

import os, sys
sys.path.append(os.path.dirname(__file__)[:-10][:-4][:-13])
#
from cognitivegeo.src.psseismic.inputoutput import inputoutput as io
from cognitivegeo.src.psseismic.analysis import analysis as ays
from cognitivegeo.src.psseismic.visualization import visualization as vis


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for editting seismic survey


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class editsurvey(object):

    survinfo = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, EditSurvey):
        EditSurvey.setObjectName("EditSurvey")
        EditSurvey.setFixedSize(400, 270)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/survey.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        EditSurvey.setWindowIcon(icon)
        self.lblsrvinfo = QtWidgets.QLabel(EditSurvey)
        self.lblsrvinfo.setObjectName("lblsrvinfo")
        self.lblsrvinfo.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lblnum = QtWidgets.QLabel(EditSurvey)
        self.lblnum.setObjectName("lblnum")
        self.lblnum.setGeometry(QtCore.QRect(120, 50, 80, 30))
        self.lblstart = QtWidgets.QLabel(EditSurvey)
        self.lblstart.setObjectName("lblstart")
        self.lblstart.setGeometry(QtCore.QRect(220, 50, 80, 30))
        self.lblstep = QtWidgets.QLabel(EditSurvey)
        self.lblstep.setObjectName("lblstep")
        self.lblstep.setGeometry(QtCore.QRect(320, 50, 40, 30))
        self.lblinl = QtWidgets.QLabel(EditSurvey)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(10, 90, 100, 30))
        self.lblxl = QtWidgets.QLabel(EditSurvey)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(10, 130, 100, 30))
        self.lblz = QtWidgets.QLabel(EditSurvey)
        self.lblz.setObjectName("lblz")
        self.lblz.setGeometry(QtCore.QRect(10, 170, 100, 30))
        self.ldtinlnum = QtWidgets.QLineEdit(EditSurvey)
        self.ldtinlnum.setObjectName("ldtinlnum")
        self.ldtinlnum.setGeometry(QtCore.QRect(120, 90, 80, 30))
        self.ldtinlstart = QtWidgets.QLineEdit(EditSurvey)
        self.ldtinlstart.setObjectName("ldtinlstart")
        self.ldtinlstart.setGeometry(QtCore.QRect(220, 90, 80, 30))
        self.ldtinlstep = QtWidgets.QLineEdit(EditSurvey)
        self.ldtinlstep.setObjectName("ldtinlstep")
        self.ldtinlstep.setGeometry(QtCore.QRect(320, 90, 40, 30))
        self.ldtxlnum = QtWidgets.QLineEdit(EditSurvey)
        self.ldtxlnum.setObjectName("ldtxlnum")
        self.ldtxlnum.setGeometry(QtCore.QRect(120, 130, 80, 30))
        self.ldtxlstart = QtWidgets.QLineEdit(EditSurvey)
        self.ldtxlstart.setObjectName("ldtxlstart")
        self.ldtxlstart.setGeometry(QtCore.QRect(220, 130, 80, 30))
        self.ldtxlstep = QtWidgets.QLineEdit(EditSurvey)
        self.ldtxlstep.setObjectName("ldtxlstep")
        self.ldtxlstep.setGeometry(QtCore.QRect(320, 130, 40, 30))
        self.ldtznum = QtWidgets.QLineEdit(EditSurvey)
        self.ldtznum.setObjectName("ldtznum")
        self.ldtznum.setGeometry(QtCore.QRect(120, 170, 80, 30))
        self.ldtzstart = QtWidgets.QLineEdit(EditSurvey)
        self.ldtzstart.setObjectName("ldtzstart")
        self.ldtzstart.setGeometry(QtCore.QRect(220, 170, 80, 30))
        self.ldtzstep = QtWidgets.QLineEdit(EditSurvey)
        self.ldtzstep.setObjectName("ldtzlstep")
        self.ldtzstep.setGeometry(QtCore.QRect(320, 170, 40, 30))
        self.btnapply = QtWidgets.QPushButton(EditSurvey)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(120, 220, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(EditSurvey)
        self.msgbox.setObjectName("msgbox")
        _center_x = EditSurvey.geometry().center().x()
        _center_y = EditSurvey.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(EditSurvey)
        QtCore.QMetaObject.connectSlotsByName(EditSurvey)


    def retranslateGUI(self, EditSurvey):
        self.dialog = EditSurvey
        #
        _translate = QtCore.QCoreApplication.translate
        EditSurvey.setWindowTitle(_translate("EditSurvey", "Edit Survey"))
        self.lblsrvinfo.setText(_translate("EditSurvey", "Survey information:"))
        self.lblnum.setText(_translate("EditSurvey", "Number"))
        self.lblnum.setAlignment(QtCore.Qt.AlignCenter)
        self.lblstart.setText(_translate("EditSurvey", "Start"))
        self.lblstart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblstep.setText(_translate("EditSurvey", "Step"))
        self.lblstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinl.setText(_translate("EditSurvey", "Inline:"))
        self.lblinl.setAlignment(QtCore.Qt.AlignRight)
        self.lblxl.setText(_translate("EditSurvey", "Crossline:"))
        self.lblxl.setAlignment(QtCore.Qt.AlignRight)
        self.lblz.setText(_translate("EditSurvey", "Time/depth:"))
        self.lblz.setAlignment(QtCore.Qt.AlignRight)
        self.ldtinlnum.setEnabled(False)
        self.ldtinlnum.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlnum.setEnabled(False)
        self.ldtxlnum.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtznum.setEnabled(False)
        self.ldtznum.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstep.setAlignment(QtCore.Qt.AlignCenter)
        self.btnapply.setText(_translate("EditSurvey", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)
        #
        if self.checkSurvInfo() is True:
            _seisinfo = self.survinfo
            self.ldtinlnum.setText(_translate("EditSurvey", str(_seisinfo['ILNum'])))
            self.ldtinlstart.setText(_translate("EditSurvey", str(_seisinfo['ILStart'])))
            self.ldtinlstep.setText(_translate("EditSurvey", str(_seisinfo['ILStep'])))
            self.ldtxlnum.setText(_translate("EditSurvey", str(_seisinfo['XLNum'])))
            self.ldtxlstart.setText(_translate("EditSurvey", str(_seisinfo['XLStart'])))
            self.ldtxlstep.setText(_translate("EditSurvey", str(_seisinfo['XLStep'])))
            self.ldtznum.setText(_translate("EditSurvey", str(_seisinfo['ZNum'])))
            self.ldtzstart.setText(_translate("EditSurvey", str(_seisinfo['ZStart'])))
            self.ldtzstep.setText(_translate("EditSurvey", str(_seisinfo['ZStep'])))


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        if self.checkSurvInfo() is False:
            vis_msg.print("ERROR in EditSurvey: No seismic survey found", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Edit Survey',
                                           'No seismic survey found')
            return
        #
        _inlnum = basic_data.str2int(self.ldtinlnum.text())
        _xlnum = basic_data.str2int(self.ldtxlnum.text())
        _znum = basic_data.str2int(self.ldtznum.text())
        _inlstart = basic_data.str2int(self.ldtinlstart.text())
        _inlstep = basic_data.str2int(self.ldtinlstep.text())
        _xlstart = basic_data.str2int(self.ldtxlstart.text())
        _xlstep = basic_data.str2int(self.ldtxlstep.text())
        _zstart = basic_data.str2int(self.ldtzstart.text())
        _zstep = basic_data.str2int(self.ldtzstep.text())
        if _inlstart is False or _inlstep is False or _inlnum is False\
            or _xlstart is False or _xlstep is False or _xlnum is False\
            or _zstart is False or _zstep is False or _znum is False:
            vis_msg.print("ERROR in EditSurvey: Non-integer survey selection", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Edit Survey',
                                           'Non-integer survey selection')
            return
        #
        _zerodata = np.zeros([_znum, _xlnum, _inlnum])
        self.survinfo = seis_ays.createSeisInfoFrom3DMat(_zerodata,
                                                         inlstart=_inlstart, inlstep=_inlstep,
                                                         xlstart=_xlstart, xlstep=_xlstep,
                                                         zstart=_zstart, zstep=_zstep
                                                         )
        #
        # QtWidgets.QMessageBox.information(self.msgbox,
        #                                   "Edit Survey",
        #                                   "Survey edited successfully")
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("EditSurvey: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Edit Survey',
            #                                'Survey not found')
            return False
        return True


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    EditSurvey = QtWidgets.QWidget()
    gui = editsurvey()
    gui.setupGUI(EditSurvey)
    EditSurvey.show()
    sys.exit(app.exec_())


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='cognitivegeo',
    version='1.1.1',
    packages=setuptools.find_packages(),
    description='A python tool for geoscience data analysis',
    author='GeoPy Team',
    author_email='geopy.info@gmail.com',
    url='https://geopyteam.wixsite.com/cognitivegeo',
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords=['Geoscience', 'Seismic'],
    install_requires=[
        'PyQt5<5.10',
        'matplotlib',
        'vispy',
        'numpy',
        'scipy',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Programming Language :: Python :: 3',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for plot seismic crossline slices


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.seismic.visualization import visualization as seis_vis
from cognitivegeo.src.vis.colormap import colormap as vis_cmap
from cognitivegeo.src.gui.configplayer import configplayer as gui_configplayer
from cognitivegeo.src.vis.messager import messager as vis_msg


QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class plotvis2dseisxl(object):

    survinfo = {}
    seisdata = {}
    plotstyle = core_set.Visual['Image']
    playerconfig = core_set.Viewer['Player']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, PlotVis2DSeisXl):
        PlotVis2DSeisXl.setObjectName("PlotVis2DSeisXl")
        PlotVis2DSeisXl.setFixedSize(500, 320)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/visxl.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVis2DSeisXl.setWindowIcon(icon)
        #
        self.lwgattrib = QtWidgets.QListWidget(PlotVis2DSeisXl)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(10, 10, 230, 240))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        #
        self.lblslice = QtWidgets.QLabel(PlotVis2DSeisXl)
        self.lblslice.setObjectName("lblslice")
        self.lblslice.setGeometry(QtCore.QRect(260, 50, 150, 30))
        self.slbslice = QtWidgets.QScrollBar(PlotVis2DSeisXl)
        self.slbslice.setObjectName("slbslice")
        self.slbslice.setOrientation(QtCore.Qt.Horizontal)
        self.slbslice.setGeometry(QtCore.QRect(260, 80, 170, 30))
        self.ldtslice = QtWidgets.QLineEdit(PlotVis2DSeisXl)
        self.ldtslice.setObjectName("ldtslice")
        self.ldtslice.setGeometry(QtCore.QRect(440, 80, 50, 30))
        self.ldtslice.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.lblcmap = QtWidgets.QLabel(PlotVis2DSeisXl)
        self.lblcmap.setObjectName("lblcmap")
        self.lblcmap.setGeometry(QtCore.QRect(260, 220, 60, 30))
        self.cbbcmap = QtWidgets.QComboBox(PlotVis2DSeisXl)
        self.cbbcmap.setObjectName("cbbcmap")
        self.cbbcmap.setGeometry(QtCore.QRect(320, 220, 120, 30))
        self.cbxflip = QtWidgets.QCheckBox(PlotVis2DSeisXl)
        self.cbxflip.setObjectName("cbxflip")
        self.cbxflip.setGeometry(QtCore.QRect(450, 220, 40, 30))
        #
        self.lblrange = QtWidgets.QLabel(PlotVis2DSeisXl)
        self.lblrange.setObjectName("lblrange")
        self.lblrange.setGeometry(QtCore.QRect(260, 130, 40, 30))
        self.ldtmin = QtWidgets.QLineEdit(PlotVis2DSeisXl)
        self.ldtmin.setObjectName("ldtmin")
        self.ldtmin.setGeometry(QtCore.QRect(300, 130, 70, 30))
        self.ldtmin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtmax = QtWidgets.QLineEdit(PlotVis2DSeisXl)
        self.ldtmax.setObjectName("ldtmax")
        self.ldtmax.setGeometry(QtCore.QRect(420, 130, 70, 30))
        self.ldtmax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblrangeto = QtWidgets.QLabel(PlotVis2DSeisXl)
        self.lblrangeto.setObjectName("lblrangeto")
        self.lblrangeto.setGeometry(QtCore.QRect(370, 130, 50, 30))
        self.lblrangeto.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.btnconfigplayer = QtWidgets.QPushButton(PlotVis2DSeisXl)
        self.btnconfigplayer.setObjectName("btnconfigplayer")
        self.btnconfigplayer.setGeometry(QtCore.QRect(390, 10, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/video.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigplayer.setIcon(icon)
        #
        self.btnplotslice = QtWidgets.QPushButton(PlotVis2DSeisXl)
        self.btnplotslice.setObjectName("btnplotslice")
        self.btnplotslice.setGeometry(QtCore.QRect(170, 270, 160, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/visxl.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplotslice.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVis2DSeisXl)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVis2DSeisXl.geometry().center().x()
        _center_y = PlotVis2DSeisXl.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVis2DSeisXl)
        QtCore.QMetaObject.connectSlotsByName(PlotVis2DSeisXl)



    def retranslateGUI(self, PlotVis2DSeisXl):
        self.dialog = PlotVis2DSeisXl
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVis2DSeisXl.setWindowTitle(_translate("PlotVis2DSeisXl", "2D Window: Seismic Crossline"))
        if self.checkSurvInfo() is True:
            _firstattrib = None
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_translate("PlotVis2DSeisXl", i))
                    self.lwgattrib.addItem(item)
                    if _firstattrib is None:
                        _firstattrib = item
            self.lwgattrib.setCurrentItem(_firstattrib)
        self.lwgattrib.itemSelectionChanged.connect(self.changeLwgAttrib)
        #
        self.lblslice.setText(_translate("PlotVis2DSeisXl", "@ crossline:"))
        if (self.checkSurvInfo() is True):
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
        else:
            _slicemin = 0
            _slicemax = 0
        self.slbslice.setMinimum(_slicemin)
        self.slbslice.setMaximum(_slicemax)
        self.slbslice.setValue(_slicemin)
        if (self.checkSurvInfo() is True):
            self.ldtslice.setText(_translate("PlotVis2DSeisXl", str(_slicemin)))
        else:
            self.ldtslice.setText(_translate("PlotVis2DSeisXl", ''))
        self.slbslice.valueChanged.connect(self.changeSlbSlice)
        self.ldtslice.textChanged.connect(self.changeLdtSlice)
        #
        self.lblcmap.setText(_translate("PlotVis2DSeisXl", "Color map:"))
        self.cbbcmap.addItems(vis_cmap.ColorMapList)
        for _i in range(len(vis_cmap.ColorMapList)):
            self.cbbcmap.setItemIcon(_i, QtGui.QIcon(
                QtGui.QPixmap(os.path.join(self.iconpath, "icons/cmap_"+vis_cmap.ColorMapList[_i]+".png")).scaled(80, 30)))
        self.cbbcmap.setCurrentIndex(list.index(vis_cmap.ColorMapList, self.plotstyle['Colormap']))
        #
        self.cbxflip.setText(_translate("PlotVis2DSeisXl", ""))
        self.cbxflip.setIcon(QtGui.QIcon(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/flip.png")).scaled(80, 80)))
        #
        self.lblrange.setText(_translate("PlotVis2DSeisXl", "Range:"))
        self.lblrangeto.setText(_translate("PlotVis2DSeisXl", "~~~"))
        if (self.checkSurvInfo() is True) \
                and (self.lwgattrib.currentItem() is not None) \
                and (self.checkSeisData(self.lwgattrib.currentItem().text()) is True):
            _min, _max = self.getAttribRange(self.lwgattrib.currentItem().text())
            self.ldtmin.setText(_translate("PlotVis2DSeisXl", str(_min)))
            self.ldtmax.setText(_translate("PlotVis2DSeisXl", str(_max)))
        #
        self.btnconfigplayer.setText(_translate("PlotVis2DSeisXl", "Slice Player"))
        self.btnconfigplayer.clicked.connect(self.clickBtnConfigPlayer)
        #
        self.btnplotslice.setText(_translate("PlotVis2DSeisXl", "Seismic Crossline Viewer"))
        self.btnplotslice.setDefault(True)
        self.btnplotslice.clicked.connect(self.clickBtnPlotSlice)


    def changeLwgAttrib(self):
        if len(self.lwgattrib.selectedItems()) > 0:
            # if len(self.lwgattrib.selectedItems()) > 1:
            #     self.ldtmin.setEnabled(False)
            #     self.ldtmax.setEnabled(False)
            # else:
            #     self.ldtmin.setEnabled(True)
            #     self.ldtmax.setEnabled(True)
            _slices = self.survinfo['XLRange'].astype(int)
            _slicemin = np.min(_slices)
            _slicemax = np.max(_slices)
            self.slbslice.setMinimum(_slicemin)
            self.slbslice.setMaximum(_slicemax)
            self.ldtslice.setText(str(self.slbslice.value()))
            _min, _max = self.getAttribRange(self.lwgattrib.currentItem().text())
            self.ldtmin.setText(str(_min))
            self.ldtmax.setText(str(_max))
        else:
            self.slbslice.setMinimum(0)
            self.slbslice.setMaximum(0)
            self.ldtslice.setText('')
            self.ldtmin.setText('')
            self.ldtmax.setText('')


    def changeSlbSlice(self):
        self.ldtslice.setText(str(self.slbslice.value()))


    def changeLdtSlice(self):
        if len(self.ldtslice.text()) > 0:
            _val = basic_data.str2int(self.ldtslice.text())
            if _val >= self.slbslice.minimum() and _val <= self.slbslice.maximum():
                self.slbslice.setValue(_val)


    def clickBtnConfigPlayer(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configplayer()
        _gui.playerconfig = {}
        _gui.playerconfig['Crossline'] = self.playerconfig
        _gui.setupGUI(_config)
        _config.exec()
        self.playerconfig = _gui.playerconfig['Crossline']
        _config.show()


    def clickBtnPlotSlice(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in PlotVis2DSeisXl: No property selected for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: Seismic Crossline',
                                           'No property selected for plot')
            return
        _sls = self.slbslice.value()
        _cmap = self.cbbcmap.currentIndex()
        _flip = self.cbxflip.isChecked()
        _min = basic_data.str2float(self.ldtmin.text())
        _max = basic_data.str2float(self.ldtmax.text())
        if _min is False or _max is False:
            vis_msg.print("ERROR in PlotVis2DSeisXl: Non-float range specified for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: Seismic Crossline',
                                           'Non-float range specified for plot')
            return
        #
        for i in range(len(_attriblist)):
            print("PlotVis2DSeisXl: Plot %d of %d properties: %s" % (i + 1, len(_attriblist), _attriblist[i].text()))
            _data = self.seisdata[_attriblist[i].text()]
            # if len(_attriblist) > 1:
            #     _min, _max = self.getAttribRange(_attriblist[i].text())
            seis_vis.plotSeisXLSlicePlayerFrom3DMat(_data, initxsl=_sls, seisinfo=self.survinfo,
                                                    titlesurf=': ' + _attriblist[i].text(),
                                                    valuemin=_min,
                                                    valuemax=_max,
                                                    colormap=vis_cmap.ColorMapList[_cmap],
                                                    flipcmap=_flip, colorbaron=True,
                                                    interpolation=self.plotstyle['Interpolation'].lower(),
                                                    playerconfig=self.playerconfig,
                                                    fontstyle=self.fontstyle,
                                                    qicon=QtGui.QIcon(os.path.join(self.iconpath, "icons/logo.png"))
                                                    )
        #
        # QtWidgets.QMessageBox.information(self.msgbox,
        #                                   "Plot Crossline",
        #                                   str(len(_attriblist)) + " properties plotted successfully")
        self.dialog.close()
        return


    def getAttribRange(self, f):
        _min = -1
        _max = 1
        if (self.checkSurvInfo() is True) \
                and (f in self.seisdata.keys()) \
                and (self.checkSeisData(f) is True):
            _min = np.min(self.seisdata[f])
            _max = np.max(self.seisdata[f])
        return _min, _max


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("PlotVis2DSeisXl: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Plot Seismic Crossline',
            #                                'Survey not found')
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    PlotVis2DSeisXl = QtWidgets.QWidget()
    gui = plotvis2dseisxl()
    gui.setupGUI(PlotVis2DSeisXl)
    PlotVis2DSeisXl.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for exporting seismic as numpy


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class exportseisnpy(object):

    survinfo = {}
    seisdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ExportSeisNpy):
        ExportSeisNpy.setObjectName("ExportSeisNpy")
        ExportSeisNpy.setFixedSize(400, 390)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/numpy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ExportSeisNpy.setWindowIcon(icon)
        #
        self.lblattrib = QtWidgets.QLabel(ExportSeisNpy)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lwgattrib = QtWidgets.QListWidget(ExportSeisNpy)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(160, 10, 230, 200))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.lbltype = QtWidgets.QLabel(ExportSeisNpy)
        self.lbltype.setObjectName("lbltype")
        self.lbltype.setGeometry(QtCore.QRect(10, 230, 150, 30))
        self.cbbtype = QtWidgets.QComboBox(ExportSeisNpy)
        self.cbbtype.setObjectName("cbbtype")
        self.cbbtype.setGeometry(QtCore.QRect(160, 230, 230, 30))
        self.lblsave = QtWidgets.QLabel(ExportSeisNpy)
        self.lblsave.setObjectName("lblsave")
        self.lblsave.setGeometry(QtCore.QRect(10, 280, 50, 30))
        self.ldtsave = QtWidgets.QLineEdit(ExportSeisNpy)
        self.ldtsave.setObjectName("ldtsave")
        self.ldtsave.setGeometry(QtCore.QRect(70, 280, 250, 30))
        self.btnsave = QtWidgets.QPushButton(ExportSeisNpy)
        self.btnsave.setObjectName("btnsave")
        self.btnsave.setGeometry(QtCore.QRect(330, 280, 60, 30))
        self.btnexportnpy = QtWidgets.QPushButton(ExportSeisNpy)
        self.btnexportnpy.setObjectName("btnexportnpy")
        self.btnexportnpy.setGeometry(QtCore.QRect(120, 330, 160, 30))
        self.btnexportnpy.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ExportSeisNpy)
        self.msgbox.setObjectName("msgbox")
        _center_x = ExportSeisNpy.geometry().center().x()
        _center_y = ExportSeisNpy.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ExportSeisNpy)
        QtCore.QMetaObject.connectSlotsByName(ExportSeisNpy)


    def retranslateGUI(self, ExportSeisNpy):
        self.dialog = ExportSeisNpy
        #
        _translate = QtCore.QCoreApplication.translate
        ExportSeisNpy.setWindowTitle(_translate("ExportSeisNpy", "Export Seismic NumPy"))
        self.lblattrib.setText(_translate("ExportSeisNpy", "Select output properties:"))
        if self.checkSurvInfo() is True:
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(_translate("ExportSeisNpy", i))
                    self.lwgattrib.addItem(item)
            self.lwgattrib.selectAll()
        self.lbltype.setText(_translate("ExportSeisNpy", "Select output type:"))
        self.cbbtype.addItems(['Dictionary', '2-D numpy matrix', '3-D numpy matrix'])
        self.cbbtype.setItemIcon(0, QtGui.QIcon(os.path.join(self.iconpath, "icons/pydict.png")))
        self.cbbtype.setItemIcon(1, QtGui.QIcon(os.path.join(self.iconpath, "icons/vis2d.png")))
        self.cbbtype.setItemIcon(2, QtGui.QIcon(os.path.join(self.iconpath, "icons/vis3d.png")))
        self.lblsave.setText(_translate("ExportSeisNpy", "Save as:"))
        self.ldtsave.setText(_translate("ExportSeisNpy", ""))
        self.btnsave.setText(_translate("ExportSeisNpy", "Browse"))
        self.btnsave.clicked.connect(self.clickBtnSave)
        self.btnexportnpy.setText(_translate("ExportSeisNpy", "Export NumPy"))
        self.btnexportnpy.clicked.connect(self.clickBtnExportSeisNpy)


    def clickBtnSave(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getSaveFileName(None, 'Select Seismic NumPy', self.rootpath,
                                        filter="Seismic NumPy files (*.seis.npy);; All files (*.*)")
        if len(_file[0]) > 0:
            self.ldtsave.setText(_file[0])


    def clickBtnExportSeisNpy(self):
        self.refreshMsgBox()
        #
        _attriblist = self.lwgattrib.selectedItems()
        if len(_attriblist) < 1:
            vis_msg.print("ERROR in ExportSeisNpy; No property selected for export", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Export Seismic NumPy',
                                           'No property selected for export')
            return
        #
        if len(self.ldtsave.text()) < 1:
            vis_msg.print("ERROR in ExportSeisNpy: No name specified for NumPy", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Export Seismic NumPy',
                                           'No name specified for NumPy')
            return
        print("ExportSeisNpy: Export %d properties" % (len(_attriblist)))
        #
        _savepath = os.path.split(self.ldtsave.text())[0]
        _savename = os.path.split(self.ldtsave.text())[1]
        #
        if len(_attriblist) > 1 and self.cbbtype.currentIndex() >= 1:
            reply = QtWidgets.QMessageBox.question(self.msgbox, 'Export Seismic Numpy',
                                                   'Warning: For exporting >=2 seismic, property name used as file name. Continue?',
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.No:
                return
        #
        if self.cbbtype.currentIndex() == 0:
            _npydata = {}
            _npydata['SeisInfo'] = self.survinfo
            for i in range(len(_attriblist)):
                _npydata[_attriblist[i].text()] = self.seisdata[_attriblist[i].text()]
            #
            np.save(os.path.join(_savepath, _savename), _npydata)
        if self.cbbtype.currentIndex() == 1:
            _npydata = seis_ays.convertSeisInfoTo2DMat(self.survinfo)
            for i in range(len(_attriblist)):
                _data = self.seisdata[_attriblist[i].text()]
                _data = np.reshape(np.transpose(_data, [2, 1, 0]), [-1, 1])
                _data = np.concatenate((_npydata, _data), axis=1)
                if len(_attriblist) > 1:
                    _savename = _attriblist[i].text() + '.seis.npy'
                #
                np.save(os.path.join(_savepath, _savename), _data)
        if self.cbbtype.currentIndex() == 2:
            reply = QtWidgets.QMessageBox.question(self.msgbox, 'Export Seismic Numpy',
                                                   'Warning: survey information not saved in 3D numpy matrix. Continue?',
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.No:
                return
            #
            for i in range(len(_attriblist)):
                _data = self.seisdata[_attriblist[i].text()]
                if len(_attriblist) > 1:
                    _savename = _attriblist[i].text() + '.seis.npy'
                #
                np.save(os.path.join(_savepath, _savename), _data)
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Export Seismic NumPy",
                                          str(len(_attriblist)) + " properties exported as NumPy successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("ExportSeisNpy: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Export Seismic NumPy',
            #                                'Survey not found')
            return False
        return True

    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)



if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ExportSeisNpy = QtWidgets.QWidget()
    gui = exportseisnpy()
    gui.setupGUI(ExportSeisNpy)
    ExportSeisNpy.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2019                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for exporting point sets as ascii file


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class exportpointsetfile(object):

    pointsetdata = {}
    #
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ExportPointSetFile):
        ExportPointSetFile.setObjectName("ExportPointSetFile")
        ExportPointSetFile.setFixedSize(400, 340)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/copy.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ExportPointSetFile.setWindowIcon(icon)
        #
        self.lblpoint = QtWidgets.QLabel(ExportPointSetFile)
        self.lblpoint.setObjectName("lblpoint")
        self.lblpoint.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lwgpoint = QtWidgets.QListWidget(ExportPointSetFile)
        self.lwgpoint.setObjectName("lwgpoint")
        self.lwgpoint.setGeometry(QtCore.QRect(160, 10, 230, 200))
        self.lwgpoint.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.lblsave = QtWidgets.QLabel(ExportPointSetFile)
        self.lblsave.setObjectName("lblsave")
        self.lblsave.setGeometry(QtCore.QRect(10, 230, 50, 30))
        self.ldtsave = QtWidgets.QLineEdit(ExportPointSetFile)
        self.ldtsave.setObjectName("ldtsave")
        self.ldtsave.setGeometry(QtCore.QRect(70, 230, 250, 30))
        self.btnsave = QtWidgets.QPushButton(ExportPointSetFile)
        self.btnsave.setObjectName("btnsave")
        self.btnsave.setGeometry(QtCore.QRect(330, 230, 60, 30))
        self.btnexportnpy = QtWidgets.QPushButton(ExportPointSetFile)
        self.btnexportnpy.setObjectName("btnexportnpy")
        self.btnexportnpy.setGeometry(QtCore.QRect(120, 280, 160, 30))
        self.btnexportnpy.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ExportPointSetFile)
        self.msgbox.setObjectName("msgbox")
        _center_x = ExportPointSetFile.geometry().center().x()
        _center_y = ExportPointSetFile.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ExportPointSetFile)
        QtCore.QMetaObject.connectSlotsByName(ExportPointSetFile)


    def retranslateGUI(self, ExportPointSetFile):
        self.dialog = ExportPointSetFile
        #
        _translate = QtCore.QCoreApplication.translate
        ExportPointSetFile.setWindowTitle(_translate("ExportPointSetFile", "Export PointSet File"))
        self.lblpoint.setText(_translate("ExportPointSetFile", "Select output pointsets:"))
        if len(self.pointsetdata.keys()) > 0:
            for i in sorted(self.pointsetdata.keys()):
                item = QtWidgets.QListWidgetItem(self.lwgpoint)
                item.setText(_translate("ExportPointSetFile", i))
                self.lwgpoint.addItem(item)
            self.lwgpoint.selectAll()
        self.lblsave.setText(_translate("ExportPointSetFile", "Save as:"))
        self.ldtsave.setText(_translate("ExportPointSetFile", ""))
        self.btnsave.setText(_translate("ExportPointSetFile", "Browse"))
        self.btnsave.clicked.connect(self.clickBtnSave)
        self.btnexportnpy.setText(_translate("ExportPointSetFile", "Export PointSet File"))
        self.btnexportnpy.clicked.connect(self.clickBtnExportPointSetFile)


    def clickBtnSave(self):
        _dialog = QtWidgets.QFileDialog()
        _file = _dialog.getSaveFileName(None, 'Select PointSet File', self.rootpath,
                                        filter="PointSet Ascii files (*.txt);; All files (*.*)")
        if len(_file[0]) > 0:
            self.ldtsave.setText(_file[0])


    def clickBtnExportPointSetFile(self):
        self.refreshMsgBox()
        #
        _pointlist = self.lwgpoint.selectedItems()
        if len(_pointlist) < 1:
            vis_msg.print("ERROR in ExportPointSetFile; No pointset selected for export", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Export PointSet FIle',
                                           'No pointset selected for export')
            return
        #
        if len(self.ldtsave.text()) < 1:
            vis_msg.print("ERROR in ExportPointSetFile: No name specified for export", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Export PointSet File',
                                           'No name specified for export')
            return
        print("ExportPointSetFile: Export %d pointsets" % (len(_pointlist)))
        #
        _savepath = os.path.split(self.ldtsave.text())[0]
        _savename = os.path.split(self.ldtsave.text())[1]
        #
        if len(_pointlist) > 1:
            reply = QtWidgets.QMessageBox.question(self.msgbox, 'Export PointSet File',
                                                   'Warning: For exporting >=2 pointset, property name used as file name. Continue?',
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.No:
                return
        #
        for i in range(len(_pointlist)):
            if len(_pointlist) > 1:
                _savename = _pointlist[i].text()
            #
            _file = open(os.path.join(_savepath, _savename), 'w')
            # write a header
            _file.write("# Headers:\n")
            if 'Inline' in self.pointsetdata[_pointlist[i].text()].keys():
                _file.write("# Inline\n")
            if 'Crossline' in self.pointsetdata[_pointlist[i].text()].keys():
                _file.write("# Crossline\n")
            if 'Z' in self.pointsetdata[_pointlist[i].text()].keys():
                _file.write("# Z\n")
            for j in sorted(self.pointsetdata[_pointlist[i].text()].keys()):
                if j != 'Inline' and j != 'Crossline' and j != 'Z':
                    _file.write("# " + j + '\n')
            # write points
            _npts = basic_mdt.maxDictConstantRow(self.pointsetdata[_pointlist[i].text()])
            for j in range(_npts):
                if 'Inline' in self.pointsetdata[_pointlist[i].text()].keys():
                    _file.write(str(self.pointsetdata[_pointlist[i].text()]['Inline'][j, 0])+'\t')
                if 'Crossline' in self.pointsetdata[_pointlist[i].text()].keys():
                    _file.write(str(self.pointsetdata[_pointlist[i].text()]['Crossline'][j, 0]) + '\t')
                if 'Z' in self.pointsetdata[_pointlist[i].text()].keys():
                    _file.write(str(self.pointsetdata[_pointlist[i].text()]['Z'][j, 0]) + '\t')
                for k in sorted(self.pointsetdata[_pointlist[i].text()].keys()):
                    if k != 'Inline' and k != 'Crossline' and k != 'Z':
                        _file.write(str(self.pointsetdata[_pointlist[i].text()][k][j, 0]) + '\t')
                _file.write('\n')
            #
            _file.close()
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Export PointSet File",
                                          str(len(_pointlist)) + " pointsets exported as Ascii File successfully")
        return


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ExportPointSetFile = QtWidgets.QWidget()
    gui = exportpointsetfile()
    gui.setupGUI(ExportPointSetFile)
    ExportPointSetFile.show()
    sys.exit(app.exec_())